--------------------------------------------------------
--  DDL for Procedure ACTIVAR_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ACTIVAR_APL" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT ACTIVAR_APL
  --
  -- ACTIVA UNA APLICACIO INACTIVA, ES A DIR, POSA DISPONIBLE UNA APLICACIO
  -- PER A QUE PUGUI SER DESCARREGADA
  --
  -- EL PROCEDIMENT CANVIARA EL PARAMETRE QUE MOSTRA LA DISPONIBILITAT DE
  -- L'APLICACIO DESPRES DE FER LES COMPROVACIONS PER A QUE NO ES PRODUEIXI
  -- CAP ERROR
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  --ERRORS POSSIBLES:
  --
  --                 VALORS NULS ALS PARAMETRES D'ENTRADA
  --                 APLICACIO JA ACTIVA
  --                 APLICACIO ERRONIA O INEXISTENT
  --                 ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  ACT APLICACIO.ACTIVA%type;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_JA_ACT  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'ACTIVAR_APL';
BEGIN
  --ES DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- ES COMPROVA QUE CAP PARAMETRE D'ENTRADA ES NULL
  IF (PARAM1 IS NULL)THEN
    RAISE EX_VAL_NUL;
  END IF;
  --ES COMPROVA SI EXISTEIX L'APLICACIO A ACTIVAR EN EL CAS QUE NO SALTARA L'EXCEPCIO NO_DATA_FOUND
  SELECT ACTIVA
  INTO ACT
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  -- SI S'HA TROBAT ES COMPROVA SI JA ESTAVA ACTIVA
  IF (ACT='A') THEN
    RAISE EX_JA_ACT;
  END IF;
  -- SI NO ESTAVA ACTIVA ES CANVIA EL PARAMETRE QUE REPRESENTA LA DISPONIBILITAT
  -- DE L'APLICACIO.
  UPDATE APLICACIO
  SET ACTIVA ='A'
  WHERE IDAPL=PARAM1;
  -- SI S'HA ARRIBAT A AQUEST PUNT VOL DIR S'HA EXECUTAT CORRECTAMENT
  --PER TANT GUARDEM A LA VARIABLE DE SORTIDA RSP EL TEXT 'OK'
  RSP:='OK';
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_JA_ACT THEN
  RSP:='ERROR: L''APLICACIO JA ESTA ACTIVA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN NO_DATA_FOUND THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ACTIVAR_APL;
/
