--------------------------------------------------------
--  DDL for Procedure ACTUALITZA_DES
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ACTUALITZA_DES" (
    PARAM1 IN NUMBER ,
    PARAM2 IN NUMBER ,
    RSP OUT VARCHAR2 )
  -- PROCEDIMENT    ACTUALITZA_DES
  --
  -- ACTUALITZA LA DESCARREGA SI HI HA UNA VERSI DIFERENT A LA JA INSTALADA
  --
  -- EL PROCEDIMENT COMPROVARA SI EL DISPOSITIU JA TE INSTALADA LA VERSIO ACTUAL
  -- DE L'APLICACIO I SI AQUESTA VERSIO ES COMPATIBLE ENCARA AMB EL DISPOSITIU
  -- TAMBE COMPROVARA QUE EL DISPOSITIU ESTA ASSIGNAT A UN USUARI.
  --
  -- UN COP FETES AQUESTES COMPROBACIONS CANVIARA A LA BASE DE DADES ELS VALORS
  -- DE VERSIO, PREU, DATA DE DESCARREGA QUE SON ELS VALORS QUE PODEN HABER CANVIAT
  -- AMB L'ACTUALITZACIO
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (IMEI DISPOSITIU), PARAM2 NUMBER (CODI APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                 VALORS NULS ALS PARAMETRES D'ENTRADA
  --                 DISPOSITIU ERRONI O INEXISTENT
  --                 APLICACIO ERRONIA O INEXISTENT
  --                 APLICACIO NO INSTALADA EN EL DISPOSITIU
  --                 DISPOSITIU NO ASSOCIAT A CAP USUARI
  --                 EL DISPOSITIU NO COMPLEIX LA RESOLUCIO MINIMA NECESSARIA
  --                 APLICACIO NO DISPONIBLE PEL SISTEMA OPERATIU DEL DISPOSITIU
  --                 APLICACIO NO DISPONIBLE AL PAIS DE L'USUARI
  --                 ALTRES
AS
  -- EXCEPCIONS
  EX_VAL_NUL    EXCEPTION;
  EX_NO_DISP    EXCEPTION;
  EX_NO_APL     EXCEPTION;
  EX_NO_INST    EXCEPTION;
  EX_DISP_NO_AS EXCEPTION;
  EX_MIN_RES    EXCEPTION;
  EX_NO_SO_APL  EXCEPTION;
  EX_NO_PREU    EXCEPTION;
  -- VARIABLES TEMPORALS
  N NUMBER;
  TIDOPER OPERADOR.IDOPER%TYPE;
  TVERSIO APLICACIO.VERSIO%TYPE;
  TPREU PREUPAIS.PREU%TYPE;
  TRXA APLICACIO.MINRESX%TYPE;
  TRYA APLICACIO.MINRESY%TYPE;
  TRXD MODEL.RESX%TYPE;
  TRYD MODEL.RESY%TYPE;
  TSO model.IDSO%type;
  DD DESCARREGA.DATDES%type;
  PA DESCARREGA.IDPAIS%TYPE;
  MM DISPOSITIU.MODELMOB%TYPE;
  TINST DESCARREGA.INST%TYPE;
  -- PARAMETRES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ACTUALITZA_DES';
  -- DEFINICIONS DELS CURSORS AMB LES CONSULTES A REALITZAR
  -- COMPROVACIO DE L'EXISTENCIA DEL DIPOSITIU
  CURSOR MODELM
  IS
    SELECT MODELMOB FROM DISPOSITIU WHERE IMEI=PARAM1 ;
  CURSOR OPER
  IS
    SELECT IDOPER
    FROM USUARI,
      DISPOSITIU
    WHERE USUARI.NUMMOB= DISPOSITIU.NUMMOB
    AND DISPOSITIU.IMEI=PARAM1;
  -- COMPROVACIO DE L'EXISTENCIA DE L'APLICACIO
  CURSOR aplicac
  IS
    SELECT VERSIO,
      MINRESX,
      MINRESY
    FROM APLICACIO
    WHERE IDAPL= PARAM2
    AND ACTIVA ='A';
  -- COMPROVACIO DE L'ESTAT DE LA DESCARREGA DE L'APLICACIO AL DISPOSITIU
  CURSOR instalat
  IS
    SELECT INST
    FROM DESCARREGA
    WHERE DESCARREGA.IMEI=PARAM1
    AND DESCARREGA.IDAPL =PARAM2 ;
  -- COMPROVACIO DE L'ESTAT DE L'APLICACIO EN REFERENCIA AL PAIS DE L'USUARI
  CURSOR PREUAPLPAIS
  IS
    SELECT PREU
    FROM PREUPAIS,
      USUARI,
      DISPOSITIU
    WHERE PREUPAIS.IDAPL =PARAM2
    AND DISPOSITIU.IMEI  =PARAM1
    AND DISPOSITIU.NUMMOB=USUARI.NUMMOB
    AND PREUPAIS.IDPAIS  =USUARI.PAIS;
BEGIN
  -- ES DESEN EL PARAMETRES PER A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2;
  --ES COMPROVA QUE CAP PARAMETRE D'ENTRADA ES NULL
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  --ES COMPROVA QUE EL DISPOSITIU EXISTEIX
  OPEN MODELM;
  FETCH MODELM INTO MM;
  IF (MODELM%NOTFOUND) THEN
    RAISE EX_NO_DISP;
  END IF;
  CLOSE MODELM;
  --ES COMPROVA QUE EL DISPOSITIU ESTA ASSOCIAT I ES DESA L'OPERADOR DEL USUARI DEL DISPOSITIU
  OPEN OPER;
  FETCH OPER INTO TIDOPER;
  IF (OPER%NOTFOUND)THEN
    RAISE EX_DISP_NO_AS;
  END IF;
  CLOSE OPER;
  --ES COMPROVA QUE L'APLICACIO ESTA DISPONIBLE PEL S.O. DEL DISPOSITIU
  SELECT IDSO
  INTO TSO
  FROM model
  WHERE MODELMOB=MM;
  SELECT COUNT(*) INTO N FROM SOAPL WHERE IDAPL=PARAM2 AND IDSO=TSO;
  IF (N=0) THEN
    RAISE EX_NO_SO_APL;
  END IF;
  -- ES COMPROVA QUE L'EXISTENCIA DE L'APLICACIO I ES DESEN DADES NECESSARIES
  OPEN APLICAC;
  FETCH aplicac INTO TVERSIO, TRXA, TRYA;
  IF (APLICAC%NOTFOUND)THEN
    RAISE EX_NO_APL;
  END IF;
  CLOSE APLICAC;
  -- ES COMPROVA QUE NO HI HA CONFLICTE AMB LES RESOLUCIONS
  SELECT RESX,
    RESY
  INTO TRXD,
    TRYD
  FROM MODEL
  WHERE MODELMOB=MM;
  IF ((TRXA     >TRXD)OR(TRYA>TRYD)) THEN
    RAISE EX_MIN_RES;
  END IF;
  -- ES COMPROVA LA DISPONIBILITAT DE L'APLICACIO AL PAIS
  OPEN PREUAPLPAIS;
  FETCH PREUAPLPAIS INTO TPREU;
  IF (PREUAPLPAIS%NOTFOUND)THEN
    RAISE EX_NO_PREU;
  END IF;
  CLOSE PREUAPLPAIS;
  --ES COMPROVA L'ESTAT DE L'APLICACIO EN REFERENCIA AL DISPOSITIU
  OPEN INSTALAT;
  FETCH INSTALAT INTO TINST;
  IF (INSTALAT%NOTFOUND) THEN
    raise ex_no_inst;
  END IF;
  -- ES DESEN DADES NECESARIES PER AL CONTROL ESTADISTIC
  SELECT DATDES,
    IDPAIS
  INTO DD,
    PA
  FROM DESCARREGA
  WHERE IMEI=PARAM1
  AND IDAPL =PARAM2;
  -- SI ESTA INSTALADA ACTUALMENT S'ACTUALITZEN LES DADES DE LA TAULA DE DESCARREGUES
  IF (TINST='I')THEN
    UPDATE DESCARREGA
    SET INST  ='I',
      DATDES  =CURRENT_DATE,
      VERSIO  =TVERSIO,
      PREU    =TPREU
    WHERE IMEI=PARAM1
    AND IDAPL =PARAM2;
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  ELSE
    -- SI NO ESTA INSTALADA SALTA L'EXCEPCIO
    RAISE EX_NO_INST;
  END IF;
  -- ES REALITZA UN CONTROL ESTADISTIC SOBRE EL PAIS DE L'USUARI L'ANY DE DESCARREGA
  -- DE L'ANTERIOR INSTALACIO DE L'APLICACIO I LA DATA ACTUAL, AQUEST CONTROL RETORNARA
  -- 'OK' SI NO HI HA HAGUT CAP ERROR.
  RSP := CONTROL_EST(extract(YEAR FROM DD),PA);
  RSP := CONTROL_EST(extract(YEAR FROM CURRENT_DATE),PA);
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS INTRODUITS AMB VALOR NUL';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DISP THEN
  RSP:='ERROR: NO EXISTEIX EL DISPOSITIU ';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PREU THEN
  RSP:='ERROR: NO ESTA DISPONIBLE L''APLICACIO AL PAIS DE L''USUARI';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_SO_APL THEN
  RSP:='ERROR: NO HI HA UNA VERSIO DE L''APLICACIO PER AL SO DEL DISPOSITIU';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DISP_NO_AS THEN
  RSP:='ERROR: NO EXISTEIX LA RELACIO ENTRE L''USUARI I EL DISPOSITIU';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: NO EXISTEIX L''APLICACIO O ESTA INACTIVA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_MIN_RES THEN
  RSP:='ERROR: EL DISPOSITIU NO COMPLEIX LA RESOLUCIO MINIMA NECESSARIA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_INST THEN
  RSP:='ERROR: APLICACIO NO INSTALADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT ALTA_DESCARREGA - '||SQLCODE||'-'||SQLERRM;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ACTUALITZA_DES;
/
