--------------------------------------------------------
--  DDL for Procedure AFEGIR_ADRE_DES
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_ADRE_DES" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT     AFEGIR_ADRE_DES
  --
  -- AFEGEIX L'ADREA INTRODUIDA AL DESENVOLUPADOR INDICAT
  --
  -- EL PROCEDIMENT COMPROVARA QUE EXISTEIX EL DESENVOLUPADOR I AFEGIRA EL VALOR
  -- D'ADREA.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI DESENVOLUPADOR), PARAM2 NVARCHAR2 (ADREA DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: CADENA DE TEXT, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                 VALORS NULS ALS PARAMETRES D'ENTRADA
  --                 DESENVOLUPADOR ERRONI O INEXISTENT
  --                 ADREA JA INTRODUIDA ANTERIORMENT
  --                 ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  ADRE DESENVOLUPADOR.ADRECADES%type;
  --EXCEPCIONS
  EX_VAL_NUL   EXCEPTION;
  EX_NO_DESENV EXCEPTION;
  EX_ADR_EX    EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'AFEGIR_ADRE_DES';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- COMPROVACIO DE L'EXISTENCIA DEL DESENVOLUPADOR I L'ADREA
  CURSOR ADREC
  IS
    SELECT ADRECADES INTO ADRE FROM DESENVOLUPADOR WHERE IDDES=PARAM1;
BEGIN
  -- ES DESEN ELS PARAMETRES PER A LA TAULA DE LOG
  ENTRADA := PARAM1||','||PARAM2;
  -- ES COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULS
  IF ((PARAM1 IS NULL) OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- ES COMPROVA QUE L'APLICACIO EXISTEIX I EN CAS AFIRMATIU ES MIRA SI JA TE UNA ADREA INTRODUIDA
  OPEN ADREC;
  FETCH ADREC INTO ADRE;
  IF (ADREC%NOTFOUND)THEN
    RAISE EX_NO_DESENV;
  END IF;
  IF (ADRE IS NOT NULL) THEN
    RAISE EX_ADR_EX;
  END IF;
  -- S'AFEGEIX L'ADREA, JA QUE L'APLICACIO EXISTEIX PERO NO TE ADREA
  UPDATE DESENVOLUPADOR
  SET ADRECADES=PARAM2
  where IDDES  =PARAM1;
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
    -- S'HA EXECUTAT CORRECTAMENT
  RSP         :='OK';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
 --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:= 'ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA ';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DESENV THEN
  RSP:= 'ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_ADR_EX THEN
  RSP:='ERROR: ADREA JA INTRODUIDA ANTERIORMENT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
--CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END AFEGIR_ADRE_DES;
/
