--------------------------------------------------------
--  DDL for Procedure AFEGIR_DESCRIPCIO
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_DESCRIPCIO" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    PARAM3 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT       AFEGIR_DESCRIPCIO
  --
  -- EL PROCEDIMENT AFEGIRA UNA DESCRIPCIO DE L'APLICACIO EN UN IDIOMA  EN CONCRET.
  -- EN EL CAS QUE AQUESTA APLICACIO JA TINGUI UNA DESCRIPCIO EN AQUELL IDIOMA AQUESTA
  -- ES SOBREESCRIU.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NVARCHAR2 (IDIOMA)
  --                       PARAM3 NVARCHAR2 (DESCRIPCIO APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: CADENA DE TEXT, NO POT SER NULL
  --
  -- VALORS DE PARAM3: CADENA DE TEXT, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_NO_APL  EXCEPTION;
  EX_VAL_NUL EXCEPTION;
  --VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'AFEGIR_DESCRIPCIO';
BEGIN
  -- ES DESEN LES VARIABLES PER A LA TAULA DE LOG
  ENTRADA := PARAM1||','||PARAM2||','||PARAM3;
  -- ES COMPROVA QUE NO HI HA CAP VALOR NUL ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL) OR (PARAM3 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- ES COMPROVA QUE L'APLICACIO EXISTEIXI
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  --S'INSERTA LA DESCRIPCIO INTRODUIDA
  INSERT
  INTO DESCRIPCIO VALUES
    (
      PARAM1,
      PARAM2,
      PARAM3
    );
        --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;

  -- LA DESCRIPCIO ESTA AFEGIDA
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_VAL_NUL THEN
  RSP:='ERROR:  VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

  --EN CAS QUE JA EXISTEIXI LA DESCRIPCIO A LA TAULA DE DESCRIPCIONS LA SOBREESCRIBIM
WHEN DUP_VAL_ON_INDEX THEN
  UPDATE DESCRIPCIO
  SET DESCRIPCIO.DESCR=PARAM3
  WHERE IDAPL         =PARAM1
  AND IDIOMA          =PARAM2;
  --S'HA PRODUIT LA INSERCIO CORRECTAMENT
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

END AFEGIR_DESCRIPCIO;
/
