--------------------------------------------------------
--  DDL for Procedure AFEGIR_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_DESENVOL" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT          AFEGIR_DESENVOL
  --
  -- AQUEST PROCEDIMENT VINCULA UNA APLICACIO AMB UN DESENVOLUPADOR
  --
  -- DESPRES DE FER LES COMPROVACIONS NECESSARIES AQUEST PROCEDIMENT AFEGEIX
  -- A UN REGISTRE DE LA TAULA DESENVOLUPADA ELS VALORS INTRODUIT PER L'USUARI
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (CODI DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALOR NUL
  --                  APLICACIO ERRONEA O INEXISTENT
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  RELACIO DESENVOLUPADOR-APLICACIO JA EXISTENT
  --                  ALTRES
AS
  --VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_NO_DES  EXCEPTION;
  EX_NO_APL  EXCEPTION;
  EX_VAL_NUL EXCEPTION;
  EX_REL_EX  EXCEPTION;
  -- VARIABLES PER LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'AFEGIR_DESENVOL';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- SELECCIONA ELS ANYS DE LES DESCARREGUES DE L'APLICACIO JA
  -- REALITZADES PER A ACTUALITZAR LES ESTADISTIQUES ANUALS 
  -- I COMPROVAR SI EL NOU DESENVOLUPADOR POT SER MAXIM EN ALGUN
  -- ANY
CURSOR EST IS SELECT DISTINCT EXTRACT (YEAR FROM DATDES) FROM DESCARREGA, DESENVOLUPADA WHERE  DESENVOLUPADA.IDAPL=DESCARREGA.IDAPL AND DESENVOLUPADA.IDDES=PARAM2;
BEGIN
  --DESEM ELS VALORS PER A LA TAULA DE LOG
  ENTRADA := PARAM1 ||','||PARAM2;
  --COMPROVEM QUE CAP VARIABLE D'ENTRADA ES NUL
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  --COMPROVEM QUE EXISTEIX EL DESENVOLUPADOR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES = PARAM2;
  IF (N       =0) THEN
    RAISE EX_NO_DES;
  END IF;
  --COMPROVEN QUE EXISTEIX L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL = PARAM1;
  IF (N       =0)THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVEM SI JA S'HAVIA INTRODUIT ABANS LA RELACIO
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADA
  WHERE IDAPL=PARAM1
  AND IDDES  =PARAM2;
  IF (N      =1) THEN
    RAISE EX_REL_EX;
  END IF;
  -- S'INSERTA LA RELACIO A LA TAULA DESENVOLUPADA
  INSERT
  INTO DESENVOLUPADA VALUES
    (
      PARAM1,
      PARAM2
    );
        --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;
  -- COMPROVEM SI EL NOU DESENVOLUPADOR ES DESENVOLUPADOR MAXIM
  -- EN ALGUN ANY PER A LES ESTADISTIQUES.
    OPEN EST;
  FETCH EST INTO N;
  WHILE (EST%FOUND)LOOP
  RSP:= CONTROL_EST2(N);
  FETCH EST INTO N;
  END LOOP;
CLOSE EST;
  --S'HA REALITZAT TOT CORRECTAMENT
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR:  VALORS NULS ALS PARAMETRES D''ENTRADA';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM L'OPERACIO
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACI ERRONIA O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT INTO PLOG VALUES
    (PROCEDIMENT, ENTRADA, RSP, CURRENT_DATE
    );  
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_REL_EX THEN
  RSP:='ERROR: RELACIO DESENVOLUPADOR APLICACI JA EXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    ); 
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END AFEGIR_DESENVOL;
/
