--------------------------------------------------------
--  DDL for Procedure AFEGIR_DISPOSITIU
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_DISPOSITIU" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           AFEGIR_DISPOSITIU
  --
  -- AQUEST PROCEDIMENT VINCULA UN DISPOSTIU MOBIL AMB UN USUARI FINAL
  --
  -- DESPRES DE FER LES COMPROVACIONS NECESARIES AQUEST PROCES LI ASSIGNA UN
  -- USUARI A UN DISPOSITIU CONCRET.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI USUARI), PARAM2 NUMBER (IMEI DISPOSITIU)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  DISPOSITIU ERRONI O INEXISTENT
  --                  USUARI ERRONI O INEXISTENT
  --                  DISPOSITIU ASSIGNAT A UN ALTRE USUARI
  --                  DISPOSITIU JA ASSIGNAT A AQUEST USUARI
  --                  ALTRES
AS
  --VARIABLES TEMPORALS
  N NUMBER;
  E NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL        EXCEPTION;
  EX_NO_DISP        EXCEPTION;
  EX_NO_USU         EXCEPTION;
  EX_DISPO_ASSIGNAT EXCEPTION;
  EX_DISPO_AS_USU   EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'AFEGIR_DISPOSITIU';
  -- DEFINICIONS DELS CURSORS DE LES CONSULTES A REALITZAR
  -- SELECCIONAR EL CODI DE L'USUARI AMB EL QUE ESTA VINCULAT EL DISPOSITIU
  CURSOR NUMM
  IS
    SELECT NUMMOB INTO E FROM DISPOSITIU WHERE IMEI = PARAM2;
BEGIN
  -- DESEM LES DADES PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  --COMPROVAR QUE NO HI HA VALORS NULS A LES VARIABLES D'ENTRADA
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  --COMPROVAR QUE EXISTEIX L'USUARI
  SELECT COUNT(*)
  INTO N
  FROM USUARI
  WHERE NUMMOB= PARAM1;
  IF (N       =0)THEN
    RAISE EX_NO_USU;
  END IF;
  -- COMPROVACIONS RELATIVES AL DISPOSITIU
  OPEN NUMM;
  FETCH NUMM INTO E;
  -- SI NO EXISTEIX EL DISPOSITIU
  IF (NUMM%NOTFOUND)THEN
    RAISE EX_NO_DISP;
  END IF;
  -- EL DISPOSITIU JA ESTA ASSIGNAT A L'USUARI
  IF (E=PARAM1) THEN
    RAISE EX_DISPO_AS_USU;
  END IF;
  -- EL DISPOSITIU ESTA ASSIGNAT (I COM JA HEM VIST QUE NO HO ESTA AL USUARI
  -- D'ENTRADA) A UN ALTRE USUARI
  IF (E IS NOT NULL) THEN
    RAISE EX_DISPO_ASSIGNAT;
  END IF;
  CLOSE NUMM;
  --S'ASSIGNA EL DISPOSITIU A L'USUARI
  UPDATE DISPOSITIU
  SET NUMMOB=PARAM1
  where IMEI=PARAM2;
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

  --S'HA REALITZAT TOT CORRECTAMENT
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM L'OPERACIO
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_DISPO_ASSIGNAT THEN
  RSP:='ERROR: DISPOSITIU JA ASSIGNAT A UN ALTRE USUARI';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_DISPO_AS_USU THEN
  RSP:='ERROR: DISPOSITIU JA ASSIGNAT A AQUEST USUARI';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_DISP THEN
  RSP:='ERROR: DISPOSITIU ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_USU THEN
  RSP:='ERROR: USUARI ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM L'OPERACIO
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  --GUARDEM L'OPERACIO
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

END AFEGIR_DISPOSITIU;
/
