--------------------------------------------------------
--  DDL for Procedure AFEGIR_PREU
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_PREU" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    PARAM3 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT      AFEGIR_PREU
  --
  -- AQUEST PROCEDIMENT AFEGEIX UN PREU D'UN APLICACIO DEPENENT DEL PAIS ON ES
  -- POT DESCARREGAR.
  --
  -- DESPRES DE REALITZAR LES COMPROVACIONS ADIENTS AFEGEIX A LA TAULA PREUPAIS
  -- EL REGISTRE AMB LES DADES D'ENTRADA.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO),
  --                       PARAM2 NVARCHAR2(2) (CODI PAIS), PARAM3 NUMBER (*,2) (PREU)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: MIDA=2, NO POT SER NULL
  --
  -- VALORS DE PARAM3: ENTER >=0 AMB 2 DECIMALS, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  PAIS ERRONI O INEXISTENT
  --                  PREU AMB VALOR NEGATIU
  --                  RELACIO PAIS-APLICACIO JA INTRODUIDA ANTERIORMENT
  --                  ALTRES
AS
  --VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_JA_INTRO EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_PAIS_ERR EXCEPTION;
  EX_PREU_NEG EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'AFEGIR_PREU';
BEGIN
  --DESEM VALORS PER A LA TAULA DE LOG
  ENTRADA :=PARAM1 ||',' ||PARAM2 ||',' ||PARAM3 ;
  --COMPROVEM QUE CAP PARAMENTRE D'ENTRADA ES NUL
  IF (((PARAM1 IS NULL)OR(PARAM2 IS NULL))OR(PARAM3 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVEM QUE EXISTEIX L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL = PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVEM QUE EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS= PARAM2;
  IF (N       =0) THEN
    RAISE EX_PAIS_ERR;
  END IF;
  -- COMPROVEM QUE SI S'HA INTRODUIT ABANS LA RELACIO PAIS-APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM PREUPAIS
  WHERE IDPAIS= PARAM2
  AND IDAPL   = PARAM1;
  IF (N       =1) THEN
    RAISE EX_JA_INTRO;
  END IF;
  -- COMPROVEM QUE EL PREU INTRODUIT SIGUI POSITIU
  IF (PARAM3<0) THEN
    RAISE EX_PREU_NEG;
  END IF;
  -- INSERTEM LA RELACIO PAIS-APLICACIO I EL VALOR D'AQUESTA AL PAIS INTRODUIT
  INSERT
  INTO PREUPAIS VALUES
    (
      PARAM1,
      PARAM2,
      PARAM3
    );
        --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;

  --S'HA REALITZAT TOT CORRECTAMENT
  RSP:='OK';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_JA_INTRO THEN
  RSP:='ERROR: PREU JA INTRODUIT';
  -- GUARDEM A AL TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_PAIS_ERR THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
     --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_PREU_NEG THEN
  RSP:='ERROR: PREU INTRODUIT AMB VALOR NEGATIU';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

END AFEGIR_PREU;
/
