--------------------------------------------------------
--  DDL for Procedure AFEGIR_SO
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_SO" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    PARAM3 IN NUMBER,
    PARAM4 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT         AFEGIR_SO
  --
  -- AQUEST PROCEDIMENT VINCULA UNA APLICACIO AMB UN SISTEMA OPERATIU MOBIL
  -- I AFEGEIX LA MIDA I L'ENLLA AL FITXER BINARI D'AQUESTA  DESPRES DE FER
  -- LES COMPROVACIONS PERTINENTS.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (CODI SO)
  --                       PARAM3 NUMBER (MIDA KB),  PARAM4 NUMBER (ENLLA AL BIN)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: ENTER > 0, NO POT SER NULL
  --
  -- VALORS DE PARAM3: ENTER > 0, NO POT SER NULL
  --
  -- VALORS DE PARAM4: CADENA DE CARACTERS, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  S.O. ERRONI O INEXISTENT
  --                  MIDA AMB VALOR NEGATIU
  --                  RELACIO APLICACIO-S.O. JA INTRODUIDA
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_NO_SO    EXCEPTION;
  EX_JA_INTRO EXCEPTION;
  EX_MIDA_NEG EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'AFEGIR_SO';
BEGIN
  --DESA VALORS PER A LA TAULA DE LOG
  ENTRADA := PARAM1||','||PARAM2||','||PARAM3 ||','||PARAM4;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF (((PARAM1 IS NULL)OR(PARAM2 IS NULL))OR((PARAM3 IS NULL)OR(PARAM4 IS NULL))) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE LA MIDA INTRODUIDA NO ESTIGUI FORA DE RANG (MIDA>0)
  IF (param3<=0) THEN
    raise ex_mida_neg;
  END IF;
  SELECT COUNT(*) INTO N FROM APLICACIO WHERE IDAPL=PARAM1;
  IF (N=0 )THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA SI EXITEIX EL SISTEMA OPERATIU
  SELECT COUNT(*)
  INTO N
  FROM SO
  WHERE IDSO=PARAM2;
  IF (N     =0) THEN
    RAISE EX_NO_SO;
  END IF;
  -- COMPROVA QUE NO S'HA INTRODUIT LA RELACIO ANTERIORMENT
  SELECT COUNT(*)
  INTO N
  FROM SOAPL
  WHERE IDAPL=PARAM1
  AND IDSO   =PARAM2;
  IF (N      =1) THEN
    RAISE EX_JA_INTRO;
  END IF;
  -- S'INSEREIX LA RELACIO EN LA BASE DE DADES
  INSERT
  INTO SOAPL VALUES
    (
      PARAM1,
      PARAM2,
      PARAM3,
      PARAM4
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

  --S'HA EXECUTAT TOT CORRECTAMENT
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO INTRODUIDA ERRONEA O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_NO_SO THEN
  RSP:='ERROR: SISTEMA OPERATIU INTRODUIT ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_JA_INTRO THEN
  RSP:='ERROR: RELACIO APLICACIO SISTEMA OPERATIU JA INTRODUIDA ANTERIORMENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN EX_MIDA_NEG THEN
  RSP:='ERROR: VALOR DE MIDA DE L''APLICACI NEGATIU';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

END AFEGIR_SO;
/
