--------------------------------------------------------
--  DDL for Procedure AFEGIR_URL_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "AFEGIR_URL_APL" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT         AFEGIR_URL_APL
  --
  -- AQUEST PROCEDIMENT AFEGEIX UN ENLLA AL VIDEO DE DEMOSTRACIO DE L'APLICACIO
  -- DESPRES D'HABER FER LES COMPROVACIONS PERTINENTS. EN CAS QUE JA S'HAGI AFEGIT
  -- S'ACTUALITZARA AL NOU ENLLA INTRODUIT.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI D'APLICACIO), PARAM2 NVARCHAR2 (URL)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: URL, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_APL  EXCEPTION;
  --VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'AFEGIR_URL_APL';
BEGIN
  --DESA VARIABLES PER A LA TAULA DE LOG
  ENTRADA :=PARAM1 ||',' ||PARAM2;
  --COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL= PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- ACTUALITZA EL VALOR DE LA URL DEL VIDEO DE DEMOSTRACIO
  UPDATE APLICACIO
  SET VIDEODEMO =PARAM2
  WHERE IDAPL   =PARAM1;
  -- S'HA EXECUTAT SENSE PROBLEMES
  RSP:='OK';
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO INTRODUIDA ERRONIA O INEXISTENT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END AFEGIR_URL_APL;
/
