--------------------------------------------------------
--  DDL for Procedure ALTA_DESCARREGA
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ALTA_DESCARREGA" (
    PARAM1 IN NUMBER ,
    PARAM2 IN NUMBER ,
    PARAM3 IN NUMBER ,
    RSP OUT VARCHAR2 )
  -- PROCEDIMENT       ALTA_DESCARREGA
  --
  -- CREA UNA DESCARREGA FETA PER UN USUARI EN UN TERMINAL MOBIL CONCRET I LA
  -- DESA A LA TAULA DESCARREGA, DESPRES DE FER LES COMPROVACIONS PERTINENTS
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (IMEI DISPOSITIU), PARAM2 NUMBER (CODI APLICACIO)
  --                       PARAM3 NUMBER (METODE PAGAMENT)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER > 0, NO POT SER NULL
  --
  -- VALORS DE PARAM2: ENTER > 0, NO POT SER NULL
  --
  -- VALORS DE PARAM3: ENTER > 0, NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  DISPOSITIU ERRONI O INEXISTENT
  --                  APLICACIO JA INSTALADA EN EL DISPOSITIU
  --                  DISPOSITIU NO ASSOCIAT A UN USUARI
  --                  DISPOSITIU NO COMPLEIX LA RESOLUCIO MINIMA
  --                  METODE DE PAGAMENT ERRONI
  --                  APLICACIO NO DISPONIBLE AL PAIS DE L'USUARI
  --                  APLICACIO NO DISPONIBLE PER AL S.O. DEL DISPOSITIU
  --                  ALTRES
AS
  -- EXCEPCIONS
  EX_VAL_NUL    EXCEPTION;
  EX_NO_DISP    EXCEPTION;
  EX_NO_APL     EXCEPTION;
  EX_JA_INST    EXCEPTION;
  EX_DISP_NO_AS EXCEPTION;
  EX_MIN_RES    EXCEPTION;
  EX_NO_MPAG    EXCEPTION;
  EX_NO_SO_APL  EXCEPTION;
  EX_NO_PREU    EXCEPTION;
  --VARIABLES TEMPORALS
  N NUMBER;
  TIDOPER OPERADOR.IDOPER%TYPE;
  TPREU PREUPAIS.PREU%TYPE;
  TPAIS PAIS.IDPAIS%TYPE;
  TVERSIO APLICACIO.VERSIO%TYPE;
  TRXA APLICACIO.MINRESX%TYPE;
  TRYA APLICACIO.MINRESY%TYPE;
  TRXD MODEL.RESX%TYPE;
  TRYD MODEL.RESY%TYPE;
  TSO MODEL.IDSO%TYPE;
  MM DISPOSITIU.MODELMOB%TYPE;
  TINST DESCARREGA.INST%TYPE;
  --VARIABLE D'IDENTIFICACIO DE CONSULTES AMB CAP RESULTAT.
  CONSULTA NUMBER;
  --VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'ALTA_DESCARREGA';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- MODEL MOVIL DEL DISPOSITIU ON ES VOL INSTALAR L'APLICACIO
  CURSOR MODELM
  IS
    SELECT MODELMOB FROM DISPOSITIU WHERE IMEI=PARAM1;
  -- OPERADOR DEL USUARI
  CURSOR OPER
  IS
    SELECT IDOPER
    FROM USUARI,
      DISPOSITIU
    WHERE USUARI.NUMMOB= DISPOSITIU.NUMMOB
    AND DISPOSITIU.IMEI=PARAM1;
  -- CONSULTA DEL PREU DE L'APLICACIO AL PAIS D'USUARI
  CURSOR PREUAPLPAIS
  IS
    SELECT PREU,
      PAIS
    FROM PREUPAIS,
      USUARI,
      DISPOSITIU
    WHERE IDAPL        =PARAM3
    AND PREUPAIS.IDPAIS=USUARI.PAIS
    AND USUARI.NUMMOB  = DISPOSITIU.NUMMOB
    AND DISPOSITIU.IMEI=PARAM1;
  -- CONSULTA D'ALTRES DADES DE L'APLICACIO
  CURSOR APLICAC
  IS
    SELECT VERSIO,
      MINRESX,
      MINRESY
    FROM APLICACIO
    WHERE IDAPL= PARAM2
    AND ACTIVA ='A';
  --CONSULTA DEL ESTAT DE L'APLICACIO EN EL DISPOSITIU INTRODUIT
  CURSOR INSTALAT
  IS
    SELECT INST FROM DESCARREGA WHERE IMEI=PARAM1 AND IDAPL=PARAM2;
BEGIN
  -- INICIALITZACIO DE LA VARIABLE D'ENTRADA PER AL LOG
  ENTRADA:=PARAM1||','||PARAM2||','||PARAM3;
  --ES COMPROVA QUE NO HI HAGI CAP VARIABLE D'ENTRADA NULLA
  IF (((PARAM1 IS NULL)OR (PARAM2 IS NULL))OR (PARAM3 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  --ES COMPROVA QUE EL DIPOSITIU EXISTEIX I ES GUARDA EL MODEL D'AQUEST
  OPEN MODELM;
  FETCH MODELM INTO MM;
  IF (MODELM%NOTFOUND)THEN
    RAISE EX_NO_DISP;
  END IF;
  CLOSE MODELM;
  --ES COMPROVA QUE EXISTEIX EL METODE DE PAGAMENT INTRODUIT
  SELECT COUNT(*)
  INTO N
  FROM MODEPAG
  WHERE MODP=PARAM3;
  IF (N     =0)THEN
    RAISE EX_NO_MPAG;
  END IF;
  -- ES COMPROVA QUE EL DISPOSITIU ESTA ASSIGNAT I S'OBTE L'OPERADOR DEL USUARI D'AQUEST
  OPEN OPER;
  FETCH OPER INTO TIDOPER;
  IF (OPER%NOTFOUND)THEN
    RAISE EX_DISP_NO_AS;
  END IF;
  CLOSE OPER;
  --ES COMPROVA QUE L'APLICACIO EXISTEIX,QUE ESTA ACTIVA I QUE LA RESOLUCIO D'AQUESTA ES MENOR QUE LA DEL DISPOSITIU
  OPEN APLICAC;
  FETCH APLICAC INTO TVERSIO,TRXA,TRYA;
  IF (APLICAC%NOTFOUND)THEN
    RAISE EX_NO_APL;
  END IF;
  CLOSE APLICAC;
  SELECT RESX,RESY INTO TRXD,TRYD FROM MODEL WHERE MODELMOB=MM;
  IF ((TRXA>TRXD)OR(TRYA>TRYD)) THEN
    RAISE EX_MIN_RES;
  END IF;
  -- ES COMPROVA QUE L'APLICACIO ESTA DISPONIBLE AL PAIS DEL USUARI I ES DESA EL PREU D'AQUESTA
  OPEN PREUAPLPAIS;
  FETCH PREUAPLPAIS INTO TPREU,TPAIS;
  IF (PREUAPLPAIS%NOTFOUND)THEN
    RAISE EX_NO_PREU;
  END IF;
  CLOSE PREUAPLPAIS;
  --ES COMPROVA QUE L'APLICACIO ESTA DISPONIBLE PEL SO DEL DISPOSITIU
  SELECT IDSO
  INTO TSO
  FROM MODEL
  WHERE MODELMOB=MM;
  SELECT COUNT(*) INTO N FROM SOAPL WHERE IDAPL=PARAM2 AND IDSO=TSO;
  IF (N=0) THEN
    RAISE EX_NO_SO_APL;
  END IF;
  --ES COMPROVA SI S'HA BAIXAT ANTERIORMENT
  OPEN INSTALAT;
  FETCH INSTALAT INTO TINST;
  IF (INSTALAT%NOTFOUND)THEN
    -- SI NO S'INSERTA PER PRIMER COP
    INSERT
    INTO DESCARREGA VALUES
      (
        PARAM1,
        PARAM2,
        PARAM3,
        TIDOPER,
        CURRENT_DATE,
        TPREU,
        TPAIS,
        TVERSIO,
        'I'
      );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
    COMMIT;
  ELSE
    -- SI S'HA INSTALAT PERO S'HA ELIMINAT DEL DISPOSITIU S'ACTUALITZA L'ESTAT
    IF (TINST='N')THEN
      UPDATE DESCARREGA
      SET INST  ='I',
        DATDES  =CURRENT_DATE,
        VERSIO  =TVERSIO,
        MODEP   =PARAM3,
        PREU    =TPREU
      WHERE IMEI=PARAM1
      AND IDAPL =PARAM2;
      --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
      COMMIT;
    ELSE
      RAISE EX_JA_INST;
    END IF;
  END IF;
  RSP:= CONTROL_EST(extract(YEAR FROM CURRENT_DATE),TPAIS);
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS INTRODUITS AMB VALOR NUL';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DISP THEN
  RSP:='ERROR: NO EXISTEIX EL DISPOSITIU ';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PREU THEN
  RSP:='ERROR: NO ESTA DISPONIBLE L''APLICACIO AL PAIS DE L''USUARI';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_MPAG THEN
  RSP:='ERROR: NO EXISTEIX EL METODE DE PAGAMENT INTRODUIT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_SO_APL THEN
  RSP:='ERROR: NO HI HA UNA VERSIO DE L''APLICACIO PER AL SO DEL DISPOSITIU';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DISP_NO_AS THEN
  RSP:='ERROR: NO EXISTEIX L''USUARI O EL DISPOSITIU ASSOCIAT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: NO EXISTEIX L''APLICACIO O ESTA INACTIVA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_JA_INST THEN
  RSP:='ERROR: APLICACIO JA INSTALADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_MIN_RES THEN
  RSP:='ERROR: EL DISPOSITIU NO COMPLEIX LA RESOLUCIO MINIMA NECESSARIA';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT ALTA_DESCARREGA - '||SQLCODE||'-'||SQLERRM;
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ALTA_DESCARREGA;
/
