--------------------------------------------------------
--  DDL for Procedure BAIXA_DESCARREGA
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "BAIXA_DESCARREGA" (
    PARAM1 IN NUMBER ,
    PARAM2 IN NUMBER ,
    RSP OUT VARCHAR2 )
  -- PROCEDIMENT       BAIXA_DESCARREGA
  --
  -- AQUEST PROCEDIMENT DONA DE BAIXA UNA DESCARREGA D'UN DISPOSITIU CONCRET,
  -- NO ELIMINA LA DESCARREGA DE LA BASE DE DADES, SI NO QUE CANVIA EL PARAMETRE
  -- INST DEL REGISTRE DE LA DESCARREGA A DONAR DE BAIXA PER A QUE QUEDI CONSTANCIA
  -- QUE L'APLICACIO JA HA SIGUT INSTALADA ANTERIORMENT.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (IMEI DISPOSITIU), PARAM2 NUMBER (CODI APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS PARAM1: NUMBER >0, NOT NULL
  --
  -- VALORS PARAM2: NUMBER >0, NOT NULL
  --
  -- VALORS RSP:  'OK'
  --              'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                   PARAMETRES D'ENTRADA AMB VALORS NULS
  --                   DISPOSITIU ERRONI O INEXISTENT
  --                   APLICACIO ERRONIA, INEXISTENT O INACTIVA
  --                   APLICACIO NO INSTALADA
  --                   DISPOSITIU NO ASSIGNAT
  --                   ALTRES
AS
  -- EXCEPCIONS
  EX_VAL_NUL    EXCEPTION;
  EX_NO_DISP    EXCEPTION;
  EX_NO_APL     EXCEPTION;
  EX_NO_INST    EXCEPTION;
  EX_NO_DISP_AS EXCEPTION;
  -- VARIABLES TEMPORALS
  N NUMBER;
  NM DISPOSITIU.NUMMOB%TYPE;
  DD DESCARREGA.DATDES%TYPE;
  P DESCARREGA.PREU%TYPE;
  PA DESCARREGA.IDPAIS%type;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'BAIXA_DESCARREGA';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- CONSULTAR EL USUARI AL QUI PERTANY EL DISPOSITIU
  CURSOR NUMM
  IS
    SELECT NUMMOB FROM DISPOSITIU WHERE IMEI=PARAM1;
BEGIN
  -- DESA LA VARIABLE PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVAR SI ELS PARAMETRES D'ENTRADA SON NULS
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVAR SI EL DISPOSITIU ESTA ASSIGNAT I EN CAS QUE HO ESTIGUI GUARDA EL
  -- CODI DEL USUARI PROPIETARI
  OPEN NUMM;
  FETCH NUMM INTO NM;
  -- NO EXISTEIX
  IF (NUMM%NOTFOUND) THEN
    RAISE EX_NO_DISP;
  END IF;
  -- NO ESTA ASSIGNAT A CAP USUARI
  IF (NM IS NULL) THEN
    RAISE EX_NO_DISP_AS;
  END IF;
  CLOSE NUMM;
  -- COMPROVA SI L'APLICACIO ESTA ACTIVA
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL= PARAM2
  AND ACTIVA ='A';
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA SI L'APLICACIO ESTA INSTALADA
  SELECT COUNT(*)
  INTO N
  FROM DESCARREGA
  WHERE IMEI= PARAM1
  AND IDAPL =PARAM2
  AND INST  ='I';
  IF (N     =0) THEN
    RAISE EX_NO_INST;
  END IF;
  -- ACTUALITZA EL VALOR QUE FA REFERENCIA A LA INSTALACIO AL VALOR QUE SIMBOLITZA
  -- "DESINSTALAT"
  UPDATE DESCARREGA
  SET INST  ='N'
  WHERE IMEI=PARAM1
  AND IDAPL =PARAM2;
  -- CONSULTA LES DADES DE LA DESCARREGA PER AL CONTROL ESTADISTIC
  SELECT DATDES,
        IDPAIS
  INTO DD,
    PA
  FROM DESCARREGA
  WHERE IMEI=PARAM1
  AND IDAPL =PARAM2;
  --REALITZA EL CONTROL ESTADISTIC
  RSP := CONTROL_EST(extract(YEAR FROM DD),PA);
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS INTRODUITS AMB VALOR NUL';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_INST THEN
  RSP:='ERROR: APLICACIO NO INSTALADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INCORRECTE';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DISP THEN
  RSP:='ERROR: DISPOSITIU ERRONI O INCORRECTE';
  -- GUARA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DISP_AS THEN
  RSP:='ERROR: DISPOSITIU NO ASSOCIAT A CAP USUARI';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END BAIXA_DESCARREGA;
/
