--------------------------------------------------------
--  DDL for Procedure CREAR_APLICACIO
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "CREAR_APLICACIO" (
    PARAM1 IN NVARCHAR2,
    PARAM2 IN NVARCHAR2,
    PARAM3 IN NUMBER,
    PARAM4 IN NUMBER ,
    RSP OUT VARCHAR2)
  -- PROCEDIMENT           CREAR_APLICACIO
  --
  -- AQUEST PROCEDIMENT GUARDA L'APLICACIO A LA BASE DE DADES
  --
  -- DESPRES DE COMPROVAR QUE NO S'HAGI INTRODUIT ANTERIORMENT I QUE LES DADES
  -- NO TENEN CAP INCONGRUENCIA LI ASSIGNA UN CODI D'IDENTIFICIACIO I L'INSEREIX
  -- A LA TAULA AMB LES SEGENTS DADES NOM APLICACIO, VERSIO I RESOLUCIO MINIMA
  -- NECESARIA
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NVARCHAR2 (NOM APLICACIO), PARAM2 NVARCHAR2 (VERSIO)
  --                       PARAM3 NUMBER (RESOLUCIO MINIMA AMPLADA), PARAM4 NUMBER (RESOLUCIO MINIMA ALADA)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: CADENA DE CARACTERS, NO NULS
  --
  -- VALOR DE PARAM2: CADENA DE CARACTERS, NO NULS
  --
  -- VALOR DE PARAM3: NUMBER>0, NO NUL
  --
  -- VALOR DE PARAM4: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                    PARAMETRES D'ENTRADA NULS
  --                    RESOLUCIO AMB VALORS NEGATIUS
  --                    APLICACIO JA EXISTENT
  --                    ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_APL_EXISTENT EXCEPTION;
  EX_VAL_NUL      EXCEPTION;
  EX_RES_NEG      EXCEPTION;
  --VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'CREAR_APLICACI';
BEGIN
  -- DESA L'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2||','||PARAM3||','||PARAM4;
  -- COMPROVA QUE NO HI HA PARAMETRES NULS D'ENTRADA
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL) OR (PARAM3 IS NULL) OR (PARAM4 IS NULL) ) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE LES RESOLUCIONS SON CORRECTES
  IF ((PARAM3<=0)OR( PARAM4<=0)) THEN
    RAISE EX_RES_NEG;
  END IF;
  --COMPROVA SI JA EXISTEIX L'APLICACIO AMB EL NOM INTRODUIT
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE NOMAPL = PARAM1;
  --SI NO EXISTEIX LA INSERTEM
  IF (N=0) THEN
    -- MIREM EL VALOR MES ALT DE CODI D'APLICACIO
    SELECT MAX(IDAPL)
    INTO N
    FROM APLICACIO;
    -- SI NO HI HA APLIACIONS A LA BASE DE DADES ASSIGNA EL VALOR 1 A L'APLICACIO
    IF (N IS NULL) THEN
      N   :=1;
      -- SI JA HI HA APLICACIONS ASSIGNA COM A CODI DE IDENTIFICACIO EL MAXIM DE TOTS +1
    ELSE
      N:=N+1;
    END IF;
    -- S'INSERTA A LA BASE DE DADES
    INSERT
    INTO APLICACIO VALUES
      (
        N,
        PARAM1,
        PARAM2,
        CURRENT_DATE,
        NULL,
        PARAM3,
        PARAM4,
        'I'
      );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
    COMMIT;
    -- MOSTREM EL CODI D'INDENTIFICACIO DE L'APLICACIO
    DBMS_OUTPUT.PUT_LINE('L''APLICACI '||PARAM1||' S''HA INSERTAT AMB NM D''IDENTIFICACI '||N );
    -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
    RSP:='OK';
    --GUARDA A LA TAULA DE LOG
    INSERT
    INTO PLOG VALUES
      (
        PROCEDIMENT,
        ENTRADA,
        RSP,
        CURRENT_DATE
      );
    --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
    COMMIT;
  ELSE
    RAISE EX_APL_EXISTENT;
  END IF;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  --GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_RES_NEG THEN
  RSP:='ERROR: VALORS DE RESOLUCI NEGATIUS';
  --GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_APL_EXISTENT THEN
  RSP:='ERROR: APLICACI EXISTENT';
  --GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END CREAR_APLICACIO;
/
