--------------------------------------------------------
--  DDL for Procedure CREAR_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "CREAR_DESENVOL" (
    PARAM1 IN NVARCHAR2,
    PARAM2 IN NVARCHAR2,
    PARAM3 IN NUMBER,
    PARAM4 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           CREAR_DESENVOL
  --
  -- AQUEST PROCEDIMENT INSEREIX UN DESENVOLUPADOR A LA BASE DE DADES
  --
  -- DESPRES DE FER LES COMPROVACIONS NECESARIES AQUEST PROCES I DESIGNAR UN
  -- CODI DE DESENVOLUPADOR, AQUEST PROCES INSEREIX EL DESENVOLUPADOR A LA
  -- TAULA
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NVARCHAR2 (NOM DESENVOLUPADOR), PARAM2 NVARCHAR2 (NOM REPRESENTANT)
  --                       PARAM3 NUMBER (TELEFON), PARAM4 NVARCHAR2 (CODI PAIS DEL DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA:  RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: CADENA DE CARACTERS, NO NULS
  --
  -- VALOR DE PARAM2: CADENA DE CARACTERS, NO NULS
  --
  -- VALOR DE PARAM3: NUMBER>0, MIDA MAX 11, NO NUL
  --
  -- VALOR DE PARAM4: CADENA DE CARACTERS (MIDA=2), NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  PARAMETRES D'ENTRADA AMB VALOR  NUL
  --                  PAIS ERRONI O INEXISTENT
  --                  DESENVOLUPADOR EXISTENT
  --                  TELEFON NEGATIU
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_DES_EX  EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  EX_TEL_NEG EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'CREAR_DESENVOL';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA := PARAM1||','||PARAM2||','||PARAM3||','||PARAM4;
  -- COMPROVA QUE CAP VALOR D'ENTRADA SIGUI NUL
  IF (((PARAM1 IS NULL)OR (PARAM2 IS NULL))OR((PARAM3 IS NULL)OR(PARAM4 IS NULL))) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EL TELEFON NO ES NEGATIU
  IF (PARAM3<=0) THEN
    RAISE EX_TEL_NEG;
  END IF;
  -- COMPROVA QUE EXISTEIX EL PAIS INTRODUIT
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM4;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- COMPROVA QUE NO S'HA AFEGIT ANTERIORMENT UN DESENVOLUPADOR AMB EL NOM INDICAT
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE NOMDES=PARAM1;
  IF (N       =1) THEN
    RAISE EX_DES_EX;
  END IF;
  -- COMPROVA SI S'HA AFEGIT JA ALGUN DESENVOLUPADOR I EN CAS AFIRMATIU EN GUARDA
  -- EL CODI D'IDENTIFICACIO MS GRAN
  SELECT MAX(IDDES)
  INTO N
  FROM DESENVOLUPADOR;
  -- SI NO HA TROBAT APLICACIONS SE LI ASSIGNA EL CODI = 1
  IF (N IS NULL) THEN
    N   :=1;
    -- SI N'HA TROBAT S'INCREMENTA EN UN EL MAXIM I  AQUEST SERA EL CODI DE IDENTIFICACIO DEL DESENOLUPADOR
  ELSE
    N:=N+1;
  END IF;
  -- INSERTA EL DESENVOLUPADOR
  INSERT
  INTO DESENVOLUPADOR VALUES
    (
      N,
      PARAM1,
      PARAM2,
      PARAM4,
      PARAM3,
      NULL
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- MOSTRA EL CODI DE IDENTICACIO ASSIGNAT A DESENVOLUPADOR
  DBMS_OUTPUT.PUT_LINE('DESENVOLUPADOR '||PARAM1||' INSERTAT AMB CODI D''IDENTIFICACIO :'||N);
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DES_EX THEN
  RSP:='ERROR: NOM DE DESENVOLUPADOR JA EXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_TEL_NEG THEN
  RSP:='ERROR: TELEFON AMB VALOR NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END CREAR_DESENVOL;
/
