--------------------------------------------------------
--  DDL for Procedure CREAR_USUARI
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "CREAR_USUARI" (
    PARAM1 IN NUMBER ,
    PARAM2 IN NVARCHAR2 ,
    PARAM3 IN NUMBER ,
    PARAM4 IN NVARCHAR2 ,
    RSP OUT VARCHAR2 )
  -- PROCEDIMENT             CREAR_USUARI
  --
  -- AQUEST PROCEDIMENT CREA UN USUARI FINAL A LA BASE DE DADES
  --
  -- DESPRES DE FER LES COMPROVACION NECESARIES, S'INSERTA L'USUARI A LA TAULA
  -- AMB LES DADES INTRODUIDES.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (NUMERO DE MOBIL), PARAM2 NVARCHAR2 (EMAIL)
  --                       PRAM3 NUMBER (CODI OPERADOR), PARAM4 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS PARAM1: NUMBER >0, MIDA MAX 11 XIFRES, NO NUL
  --
  -- VALORS PARAM2: CADENA DE CARACTER, NO NUL
  --
  -- VALORS PARAM3: NUMBER > 0, NO NUL
  --
  -- VALORS PARAM4: CADENA DE CARACTER, MIDA=2, NO NUL
  --
  -- VALORS RSP:    'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  USUARI JA INTRODUIT ANTERIORMENT
  --                  OPERADOR ERRONI O INEXISTENT
  --                  PAIS ERRONI O INEXISTENT
  --                  NUMERO DE MOBIL INTRODUIT NEGATIU
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER ;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_USU_EX  EXCEPTION;
  EX_NO_OPER EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  EX_NUM_NEG EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'CREAR_USUARI';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2||','||PARAM3||','||PARAM4;
  -- COMPROVA QUE TOTS EL PARAMETRES D'ENTRADA NO SIGUIN NULS
  IF(((PARAM1 IS NULL)OR(PARAM2 IS NULL))OR((PARAM3 IS NULL)OR(PARAM4 IS NULL))) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EL NUMERO DE MOBIL INTRODUIT NO SIGUI NEGATIU
  IF (PARAM1<=0) THEN
    RAISE EX_NUM_NEG;
  END IF;
  -- COMPROVA QUE NO S'HAGI INTRODUIT JA UN USUARI AMB EL NUMERO DE MOBIL D'ENTRADA
  SELECT COUNT(*)
  INTO N
  FROM USUARI
  WHERE NUMMOB=PARAM1;
  IF (N       =1) THEN
    RAISE EX_USU_EX ;
  END IF;
  -- COMPROVA QUE EXISTEIX L'OPERADOR
  SELECT COUNT(*)
  INTO N
  FROM OPERADOR
  WHERE IDOPER=PARAM3;
  IF (N       =0) THEN
    RAISE EX_NO_OPER;
  END IF;
  -- COMPROVA QUE EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM4;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- INSERTA L'USUARI A LA TAULA
  INSERT
  INTO USUARI VALUES
    (
      PARAM1,
      PARAM2,
      PARAM3,
      PARAM4
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO Plog VALUES
    (
      procediment,
      entrada,
      rsp,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;
--TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NUM_NEG THEN
  RSP:='ERROR:  NUMERO DE MOBIL INTRODUIT NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_USU_EX THEN
  RSP:='ERROR:  USUARI JA EXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_OPER THEN
  RSP:='ERROR: OPERADOR ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END CREAR_USUARI;
/
