--------------------------------------------------------
--  DDL for Procedure DESACTIVA_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "DESACTIVA_APL" (
    param1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT               DESACTIVA_APL
  --
  -- AQUEST PROCEDIMENT POSA EN ESTAT D'INACTIVITAT L'APLICACIO INTRODUIDA.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  L'APLICACIO JA ESTA INACTIVA
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  ACT APLICACIO.ACTIVA%type;
  -- EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_JA_INACT EXCEPTION;
  -- VARIABLES PER A LA TAULA D'ENTRADA
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'DESACTIVA_APL';
BEGIN
  -- DESA EL PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- COMPROVA QUE NO HI HA VALORS NULS ALS PARAMETRES D'ENTRADA
  IF (PARAM1 IS NULL)THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE L'APLICACIO INTRODUIDA EXISTEIXI I QUE NO ESTIGUI INACTIVA JA
  SELECT ACTIVA
  INTO ACT
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (ACT    ='I') THEN
    RAISE EX_JA_INACT;
  END IF;
  -- ACTUALITZA EL PARAMETRE D'ACTIVITAT DE L'APLICACIO
  UPDATE APLICACIO
  SET ACTIVA ='I'
  WHERE IDAPL=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT EXCEPCIONS
EXCEPTION
WHEN EX_JA_INACT THEN
  RSP:='ERROR: L''APLICACIO JA ESTA INACTIVA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN NO_DATA_FOUND THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  INSERT INTO PLOG VALUES
    ( PROCEDIMENT, ENTRADA,RSP,CURRENT_DATE
    );
  DBMS_OUTPUT.PUT_LINE(RSP);
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END DESACTIVA_APL;
/
