--------------------------------------------------------
--  DDL for Procedure DES_MOD_PAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "DES_MOD_PAIS" (
    PARAM1 IN NUMBER ,
    PARAM2 IN NVARCHAR2 ,
    RSP OUT NVARCHAR2 )
  -- PROCEDIMENT               DES_MOD_PAIS
  --
  -- AQUEST PROCEDIMENT MODIFICA EL PAIS AL QUAL ESTA REGISTRAT UN DESENVOLUPADOR
  -- CONCRET.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI DESENVOLUPADOR), PARAM2 NVARCHAR2 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DE PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER >0 , NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  PAIS ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_DES  EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'DES_MOD_PAIS';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL) ) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX EL DESENVOLUPADOR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_DES;
  END IF;
  -- COMPROVA QUE EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM2;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- ACTUALTIZA EL DESENVOLUPADOR
  UPDATE DESENVOLUPADOR
  SET IDPAIS =PARAM2
  WHERE IDDES=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END DES_MOD_PAIS;
/
