--------------------------------------------------------
--  DDL for Procedure ELIMINAR_DISPOSITIU
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIMINAR_DISPOSITIU" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT             ELIMINAR_DISPOSITIU
  --
  -- AQUEST PROCEDIMENT DESVINCULA EL DISPOSITIU INDICAT DE L'USUARI I DESINSTALA
  -- LES APLICACIONS DESCARREGADES EN AQUEST DISPOSITIU
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (NUMERO USUARI), PARAM2 NUMBER (IMEI DISPOSITIU)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, MIDA MAX=11 XIFRES, NO NUL
  --
  -- VALOR DE PARAM2: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  DISPOSITIU ERRONI O INEXISTENT
  --                  USUARI ERRONI O INEXISTENT
  --                  DISPOSITIU NO ASSIGNAT
  --                  DISPOSITIU NO ASSIGNAT A AQUEST USUARI
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  E NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL           EXCEPTION;
  EX_NO_DISP           EXCEPTION;
  EX_NO_USU            EXCEPTION;
  EX_DISPO_NO_ASSIGNAT EXCEPTION;
  EX_DISPO_NO_AS_USU   EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIMINAR_DISPOSITIU';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- SELECCIONAR USUARI PROPIETARI DEL DISPOSITIU
  CURSOR NUMM
  IS
    SELECT NUMMOB FROM DISPOSITIU WHERE IMEI=PARAM2;
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2;
  --COMPROVA SI HI HA VALORS NULS ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL))THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI EXISTEIX L'USUARI
  SELECT COUNT(*)
  INTO N
  FROM USUARI
  WHERE NUMMOB=PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_USU;
  END IF;
  -- CONSULTEM EL PROPIETARI DEL DISPOSITIU
  OPEN NUMM;
  FETCH NUMM INTO E;
  -- EL DISPOSITIU NO ES TROBA A LA BASE DE DADES
  IF (NUMM%NOTFOUND) THEN
    RAISE EX_NO_DISP;
  END IF;
  -- EL DISPOSITIU NO TE PROPIETARI
  IF (E IS NULL) THEN
    RAISE EX_DISPO_NO_ASSIGNAT;
  END IF;
  -- EL PROPIETARI ES DIFERENT DEL USUARI INTRODUIT
  IF (E<>PARAM1) THEN
    RAISE EX_DISPO_NO_AS_USU;
  END IF;
  -- ACTUALITZA EL DISPOSITIU ANULANT EL VALOR DE NUMERO DE MOBIL
  UPDATE DISPOSITIU
  SET NUMMOB=NULL
  WHERE IMEI=PARAM2
  AND NUMMOB=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- DESINSTALA LES DESCARREGUES D'AQUEST DISPOSITIU
  UPDATE DESCARREGA
  SET INST  ='N'
  WHERE IMEI=PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DISPO_NO_ASSIGNAT THEN
  RSP:='ERROR: DISPOSITIU NO ESTA ASSIGNAT A CAP USUARI';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DISPO_NO_AS_USU THEN
  RSP:='ERROR: DISPOSITIU NO ESTA ASSIGNAT A AQUEST USUARI';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DISP THEN
  RSP:='ERROR: DISPOSITIU INTRODUIT INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_USU THEN
  RSP:='ERROR: USUARI INTRODUIT INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIMINAR_DISPOSITIU;
/
