--------------------------------------------------------
--  DDL for Procedure ELIMINAR_USUARI
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIMINAR_USUARI" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT       ELIMINAR_USUARI
  --
  -- AQUEST PROCEDIMENT ELIMINA L'USURI AMB EL NUMERO DE MOBIL INTRODUIT I EN
  -- DESVINCULA ELS DISPOSITIUS QUE AQUEST PUGUI TENIR, I EN DESINSTALA LES
  -- APLICACIONS INSTALADES EN AQUESTS.
  --
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (NUMERO USUARI)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, MIDA MAX=11 XIFRES, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  USUARI ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  ANYC NUMBER;
  PAISU PAIS.IDPAIS%TYPE;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_USU  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIMINAR_USUARI';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- CONSULTOR PER AL CONTROL ESTADISTIC
 
 CURSOR AE IS SELECT DISTINCT EXTRACT(YEAR FROM DATDES), IDPAIS FROM DESCARREGA WHERE IMEI IN (SELECT IMEI FROM DISPOSITIU WHERE NUMMOB=PARAM1); 
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI EXISTEIX L'USUARI
  SELECT COUNT(*)
  INTO N
  FROM USUARI
  WHERE NUMMOB=PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_USU;
  END IF;
  -- DESINSTALA LES DESCARREGUES DELS DISPOSITIUS DEL USUARI
  UPDATE DESCARREGA
  SET INST    ='N'
  WHERE IMEI IN
    (SELECT IMEI FROM DISPOSITIU WHERE DISPOSITIU.NUMMOB=PARAM1
    );
  -- ACTUALITZEM LES TAULES ESTADISTIQUES
 OPEN AE;
 FETCH AE INTO ANYC, PAISU;
wHILE (AE%FOUND) LOOP
 rsp:=control_est(ANYC,PAISU);
 FETCH AE INTO ANYC, PAISU;
 END LOOP;
CLOSE AE;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- DESVINCULA ELS DISPOSITIUS DEL USUARI
  UPDATE DISPOSITIU
  SET NUMMOB  =NULL
  WHERE NUMMOB=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- ELIMINA L'USUARI
  DELETE FROM USUARI WHERE NUMMOB= PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
-- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_USU THEN
  RSP:='ERROR: USUARI ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIMINAR_USUARI;
/
