--------------------------------------------------------
--  DDL for Procedure ELIM_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_APL" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT              ELIM_APL
  --
  -- AQUEST PROCEDIMENT ELIMINA L'APLICACIO DE LA BASE DE DADES I TOTES LES DADES
  -- RELACIONADES AMB AQUESTA
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DE L'OPERACIO)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  ANYC NUMBER;
  PAISD PAIS.IDPAIS%TYPE;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_APL  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_APL';
  -- CURSORS AMB CONSULTES A REALITZAR
  -- CONSULTA PER AL CONTROL ESTADISTIC
  CURSOR AE IS SELECT DISTINCT EXTRACT(YEAR FROM DATDES), IDPAIS FROM DESCARREGA WHERE IDAPL=PARAM1;
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIXI L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0)THEN
    RAISE EX_NO_APL;
  END IF;
  -- DONA DE BAIXA LES DESCARREGUES FETES PER A PODER FER EL CONTROL ESTADISTIC
  UPDATE DESCARREGA SET INST='N' WHERE IDAPL=PARAM1;
-- FA CONTROL ESTADISTIC
OPEN AE;
FETCH AE INTO ANYC,PAISD;
WHILE (AE%FOUND) LOOP
RSP:=CONTROL_EST(ANYC,PAISD);
FETCH AE INTO ANYC,PAISD;
END LOOP;

  -- ESBORRA L'APLICACIO DE LA TAULA
  DELETE FROM APLICACIO WHERE IDAPL=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO INTRODUIDA ERRONEA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_APL;
/
