--------------------------------------------------------
--  DDL for Procedure ELIM_DESCRIP
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_DESCRIP" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT          ELIM_DESCRIP
  --
  -- AQUEST PROCEDIMENT ELIMINA UNA DESCRIPCIO D'UNA APLICACIO DONAT L'IDIOMA I
  -- EL CODI D'AQUEST
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NVARCHAR2 (IDIOMA)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE PARAM2: CADENA DE CARACTERS, NO NUL
  --
  -- VALOR DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  DESCRIPCIO EN L'IDIOMA INTRODUIT INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_APL  EXCEPTION;
  EX_NO_DESC EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_DESCRIP';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIXI L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA QUE EXISTEIXI LA DESRIPCIO DE L'APLICACIO EN L'IDIOMA INDICAT
  SELECT COUNT(*)
  INTO N
  FROM DESCRIPCIO
  WHERE IDAPL=PARAM1
  AND IDIOMA =PARAM2;
  IF (N      =0) THEN
    RAISE EX_NO_DESC;
  END IF;
  -- S'ESBORRA LA DESCRIPCIO DE LA BASE DE DADES
  DELETE
  FROM DESCRIPCIO
  WHERE IDAPL=PARAM1
  AND IDIOMA =PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTA EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DESC THEN
  RSP:='ERROR: IDIOMA ERRONI O DESCRIPCIO INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_DESCRIP;
/
