--------------------------------------------------------
--  DDL for Procedure ELIM_DESENV
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_DESENV" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT        ELIM_DESENV
  --
  -- AQUEST PROCEDIMENT DESVINCULA UN DESENVOLUPADOR D'UNA APLICACIO DESPRES DE
  -- FER LES COMPROVACIONS ADIENTS
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (CODI DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE PARAM2: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  RELACIO DESENVOLUPADOR-APLICACIO INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_APL  EXCEPTION;
  EX_NO_DES  EXCEPTION;
  EX_NO_REL  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  pROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_DESENV';
 -- CURSORS AMB LES CONSULTES A REALITZAR
 -- SELECCIONA ELS ANYS ON EL DESENVOLUPADOR A DESVINCULAR ES MAXIM
 -- PER AL CONTROL ESTADISTIC
  CURSOR EST IS SELECT ANYC FROM EST_ANUAL WHERE DESENVOLUPADOR=PARAM2;
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA QUE NO HI HAGI VALORS NULS ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0)THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA QUE EXISTEIX EL DESENVOLUPADOR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES=PARAM2;
  IF (N      =0) THEN
    RAISE EX_NO_DES;
  END IF;
  -- COMPROVA QUE LA RELACIO DESENVOLUPADOR-APLICACIO EXISTEIX
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADA
  WHERE IDAPL=PARAM1
  AND IDDES  =PARAM2;
  IF (N      =0) THEN
    RAISE EX_NO_REL;
  END IF;
  -- ESBORRA LA RELACIO DE LA TAULA
  DELETE
  FROM DESENVOLUPADA
  WHERE IDAPL=PARAM1
  AND IDDES  =PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
-- ES CERQUEN A LA TAULA ESTADISTICA ANUAL, ELS ANYS EN ELS QUALS EL DESENVOLUPADOR
-- INTRODUIT ERA MAXIM I S'ACTUALITZA L'ESTADISTICA ANUAL
-- D'AQUELLS ANYS
   OPEN EST;
  FETCH EST INTO N;
  WHILE (EST%FOUND)LOOP
  RSP:= CONTROL_EST2(N);
  FETCH EST INTO N;
  END LOOP;
CLOSE EST;

  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_REL THEN
  RSP:='ERROR: RELACIO DESENVOLUPADOR - APLICACIO INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_DESENV;
/
