--------------------------------------------------------
--  DDL for Procedure ELIM_DESENVOL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_DESENVOL" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           ELIM_DESENVOL
  --
  -- AQUEST PROCEDIMENT ELIMINA EL DESENVOLUPADOR DE LA BASE DE DADES, DESPRES
  -- DE FER LES COMPROVACIONS NECESSARIES
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_DES  EXCEPTION;
  -- PARAMETRES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_DESENVOL';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- SELECCIONA ELS ANYS ON EL DESENVOLUPADOR S'HA ESBORRAT 
  -- PER AL CONTROL ESTADISTIC
  CURSOR EST IS SELECT ANYC FROM EST_ANUAL WHERE DESENVOLUPADOR IS NULL;
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA := PARAM1;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF (PARAM1 IS NULL)THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX EL DESENVOLUPADOR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_DES;
  END IF;
  -- ESBORREM EL DESENVOLUPADOR
  DELETE FROM DESENVOLUPADOR WHERE IDDES=PARAM1 ;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- ES CERQUEN A LA TAULA ESTADISTICA ANUAL, ELS ANYS EN ELS QUALS EL DESENVOLUPADOR
  -- MAXIM ES NULL JA QUE VOLDRA DIR QUE EL DESENVOLUPADOR D'ENTRADA   -- ERA MAXIM I S'HA ELIMINAT I S'ACTUALITZA L'ESTADISTICA ANUAL
-- D'AQUELLS ANYS
   OPEN EST;
  FETCH EST INTO N;
  WHILE (EST%FOUND)LOOP
  RSP:= CONTROL_EST2(N);
  FETCH EST INTO N;
  END LOOP;
CLOSE EST;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_DESENVOL;
/
