--------------------------------------------------------
--  DDL for Procedure ELIM_PREU
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_PREU" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT       ELIM_PREU
  --
  -- AQUEST PROCEDIMENT ELIMINA EL PREU D'UNA APLICACIO EN UN PAIS DETERMINAT
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NVARCHAR2 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE PARAM2: CADENA DE CARACTERS, MIDA=2 , NO NUL
  --
  -- VALOR DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  PAIS ERRONI O INEXISTENT
  --                  PREU NO INTRODUIT ANTERIORMENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_NO_INTRO EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_PAIS_ERR EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_PREU';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1 ||',' ||PARAM2 ;
  -- COMPROVA QUE NO HI HAGI VALORS NULS ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX L'APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL = PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMROVA QUE EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS= PARAM2;
  IF (N       =0) THEN
    RAISE EX_PAIS_ERR;
  END IF;
  -- COMPROVA SI S'HA INSERTAT EL PREU ANTERIORMENT
  SELECT COUNT(*)
  INTO N
  FROM PREUPAIS
  WHERE IDPAIS= PARAM2
  AND IDAPL   = PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_INTRO;
  END IF;
  -- ESBORRA EL PREU DE L'APLICACIO EN EL PAIS DONAT DE LA BASE DE DADES
  DELETE
  FROM PREUPAIS
  WHERE IDAPL= PARAM1
  AND IDPAIS =PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_INTRO THEN
  RSP:='ERROR: PREU NO INTRODUIT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_PAIS_ERR THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
 -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_PREU;
/
