--------------------------------------------------------
--  DDL for Procedure ELIM_SO
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ELIM_SO" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    RSP OUT nvarchar2)
    -- PROCEDIMENT            ELIM_SO
    --
    -- AQUEST PROCEDIMENT S'ENCARREGA DE ESBORRAR LA DISPONIBILITAT D'UNA APLICACIO
    -- EN EL SISTEMA OPERATIU INDICAT, UN COP FETES COMPROVACIONS SOBRE ELS CAMPS
    -- D'ENTRADA.
    --
    -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (CODI S.O.)
    -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALOR DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALOR DE PARAM2: NUMBER>0, NO NUL
  --
  -- VALOR DE RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  S.O. ERRONI O INEXISTENT
  --                  RELACIO S.O.-APLICACIO INEXISTENT
  --                  ALTRES
as
-- VARIABLES TEMPORALS
  N           number;
  -- EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_NO_SO    EXCEPTION;
  EX_NO_INTRO EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ELIM_SO';
begin
-- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA     := PARAM1 ||',' ||PARAM2 ;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  end if;
  -- COMPROVA SI EXISTEIX L'APLICACIO
  SELECT COUNT(*) INTO N FROM APLICACIO WHERE IDAPL=PARAM1;
  IF (N=0 )THEN
    RAISE EX_NO_APL;
  end if;
  -- COMPROVA QUE EXISTEIX EL S.O.
  SELECT COUNT(*) INTO N FROM SO WHERE IDSO=PARAM2;
  IF (N=0) THEN
    RAISE EX_NO_SO;
  end if;
  -- COMPROVA LA DISPONIBILITAT DE L'APLICACIO EN EL S.O. INDICAT
  SELECT COUNT(*) INTO N FROM SOAPL WHERE IDAPL=PARAM1 AND IDSO=PARAM2;
  IF (N=0) THEN
    RAISE EX_NO_INTRO;
  end if;
  -- ESBORRA LA RELACIO ENTRE EL S.O. I L'APLICACIO
  delete from SOAPL where IDAPL=PARAM1 and IDSO= PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;
  -- S'HA REALITZAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
 -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO INTRODUIDA ERRONEA O INEXISTENT';
 -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_SO THEN
  RSP:='ERROR: SISTEMA OPERATIU INTRODUIT ERRONI O INEXISTENT';
-- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_INTRO THEN
  RSP:='ERROR: RELACIO APLICACIO SISTEMA OPERATIU NO INTRODUIDA ANTERIORMENT';
 -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
 -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ELIM_SO;
/
