--------------------------------------------------------
--  DDL for Procedure MOD_ADRE_DESENV
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MOD_ADRE_DESENV" (
    param1 IN NUMBER,
    param2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT          MOD_ADRE_DESENV
  --
  -- AQUEST PROCEDIMENT MODIFICA L'ADREA FISICA DE LES OFICINES DEL DESENVOLUPADOR
  -- DESPRES DE REALITZAR LES COMPROVACION NECESARIES
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI DESENVOLUPADOR), PARAM2 NVARCHAR2 (NOVA ADREA)
  -- PARAMETRES DE SORTIDA:  RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_DES  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'MOD_ADRE_DESENV';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI EXISTEIX EL DESENVOLUPADOR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_DES;
  END IF;
  -- ACTUALITZA L'ADREA DEL DESENVOLUPADOR INDICAT.
  UPDATE DESENVOLUPADOR
  SET ADRECADES=PARAM2
  WHERE IDDES  =PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END MOD_ADRE_DESENV;
/
