--------------------------------------------------------
--  DDL for Procedure MOD_DES_NOM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MOD_DES_NOM" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT            MOD_DES_NOM
  --
  -- AQUEST PROCEDIMENT PERMET CANVIAR EL NOM D'UN DESENVOLUPADOR DONANT EL CODI
  -- D'IDENTIFICACIO D'AQUEST, SEMPRE I QUAN EL NOM NO ESTIGUI JA A LA BASE DE
  -- DADES
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI DESENVOLUPADOR), PARAM2 NVARCHAR2 (NOU NOM DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  NOM DE DESENVOLUPADOR JA EXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_DES  EXCEPTION;
  EX_DES_EX  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'MOD_DES_NOM';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF ((PARAM1 IS NULL)OR(PARAM2 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI EXISTEIX EL DESENVOLUPADOR AMB CODI=PARAM1
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE IDDES=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_DES;
  END IF;
  -- COMPROVA SI EXISTEIX ALGUN DESENVOLUPADOR AMB EL NOU NOM A ACTUALITZAR
  SELECT COUNT(*)
  INTO N
  FROM DESENVOLUPADOR
  WHERE NOMDES=PARAM2;
  IF (N       =1) THEN
    RAISE EX_DES_EX;
  END IF;
  -- S'ACTUALITZA EL NOM DEL DESENVOLUPADOR
  UPDATE DESENVOLUPADOR
  SET NOMDES =PARAM2
  WHERE IDDES=PARAM1;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  INSERT INTO PLOG VALUES
    ( PROCEDIMENT, ENTRADA , RSP, CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  INSERT INTO PLOG VALUES
    ( PROCEDIMENT, ENTRADA , RSP, CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_DES_EX THEN
  RSP:='ERROR: NOM DE DESENVOLUPADOR JA EXISTENT';
  INSERT INTO PLOG VALUES
    ( PROCEDIMENT, ENTRADA , RSP, CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||procediment||' - '||SQLCODE||'-'||SQLERRM;
  INSERT INTO PLOG VALUES
    ( PROCEDIMENT, ENTRADA , RSP, CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END MOD_DES_NOM;
/
