--------------------------------------------------------
--  DDL for Procedure MOD_MIDA_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MOD_MIDA_APL" (
    PARAM1 IN NUMBER,
    PARAM2 IN NUMBER,
    PARAM3 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT         MOD_MIDA_APL
  --
  -- AQUEST PROCEDIMENT MODIFICA LA MIDA D'UNA APLICACIO DONAT EL CODI D'AQUESTA,
  -- LA NOVA MIDA I EL S.O. EN EL QUAL MEDIRA AQUEST ESPAI, SEMPRE QUE NO HI HAGI
  -- CAP ERROR.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (CODI SO)
  --                       PARAM3 NUMBER (NOVA MIDA)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALORS DE PARAM2: NUMBER>0, NO NUL
  --
  -- VALORS DE PARAM3: NUMBER>0, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  S.O. ERRONI O INEXISTENT
  --                  MIDA D'APLICACIO AMB VALOR NEGATIU
  --                  RELACIO APLICACIO - SISTEMA OPERATIU INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'MOD_MIDA_APL';
  -- EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_NO_SO    EXCEPTION;
  EX_MIDA_NEG EXCEPTION;
  EX_NO_REL   EXCEPTION;
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2||','||PARAM3;
  -- COMPROVA SI HI HA VALORS NULS ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL) OR (PARAM2 IS NULL) OR (PARAM3 IS NULL) )THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI LA MIDA INTRODUIDA ES NEGATIVA
  IF (PARAM3 <=0) THEN
    RAISE EX_MIDA_NEG;
  END IF;
  -- COMPROVA SI L'APLICACIO EXISTEIX
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA SI EL S.O. EXISTEIX
  SELECT COUNT(*)
  INTO N
  FROM SO
  WHERE IDSO=PARAM2;
  IF (N     =0) THEN
    RAISE EX_NO_SO;
  END IF;
  -- COMPROVA SI EXISTEIX LA RELACIO S.O.-APLICACIO
  SELECT COUNT(*)
  INTO N
  FROM SOAPL
  WHERE IDAPL=PARAM1
  AND IDSO   =PARAM2;
  IF (N      =0) THEN
    RAISE EX_NO_REL;
  END IF;
  -- ACTUALITZA LA RELACIO S.O.-APLICACIO I LA MIDA DE L'APLICACIO
  UPDATE SOAPL
  SET MIDA   =PARAM3
  WHERE IDAPL=PARAM1
  AND IDSO   =PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_SO THEN
  RSP:='ERROR: SISTEMA OPERATIU ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_REL THEN
  RSP:='ERROR: RELACIO APLICACIO - SISTEMA OPERATIU INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_MIDA_NEG THEN
  RSP:='ERROR: MIDA INTRODUIDA AMB VALOR NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END MOD_MIDA_APL;
/
