--------------------------------------------------------
--  DDL for Procedure MOD_PREU_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MOD_PREU_APL" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    PARAM3 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           MOD_PREU_APL
  --
  -- AQUEST PROCEDIMENT MODIFICA EL PREU D'UNA APLICACIO EN UN PAIS DETERMINAT
  -- SEMPRE I QUAN AQUESTA RELACIO PAIS-APLICACIO HAGI ESTAT INTRODUIDA ANTERIORMENT
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NVARCHAR2 (CODI PAIS)
  --                       PARAM3 NUMBER (NOU PREU)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS DE PARAM3: NUMBER(*,2)>0, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  PAIS ERRONI O INEXISTENT
  --                  PREU D'APLICACIO AMB VALOR NEGATIU
  --                  RELACIO APLICACIO - PAIS INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL  EXCEPTION;
  EX_PREU_NEG EXCEPTION;
  EX_PAIS_ERR EXCEPTION;
  EX_NO_APL   EXCEPTION;
  EX_NO_REL   EXCEPTION;
  -- PARAMETRES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'MOD_PREU_APL';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2||','||PARAM3;
  -- COMPROVA SI HI HA ALGUN PARAMETRE D'ENTRADA AMB VALOR NUL
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL) OR (PARAM3 IS NULL)) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI EL PREU ES NEGATIU
  IF (PARAM3<0) THEN
    RAISE EX_PREU_NEG;
  END IF;
  -- COMPROVA SI EXISTEIX EL PAIS INTRODUIT
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS = PARAM2;
  IF (N        =0) THEN
    RAISE EX_PAIS_ERR;
  END IF;
  -- COMPROVA SI EXISTEIX L'APLICACIO INTRODUIDA
  SELECT COUNT(*)
  INTO N
  FROM APLICACIO
  WHERE IDAPL=PARAM1;
  IF (N      =0) THEN
    RAISE EX_NO_APL;
  END IF;
  -- COMPROVA SI S'HA INTRODUIT ANTERIORMENT EL PREU A L'APLICACIO EN EL PAIS DETERMINAT
  SELECT COUNT(*)
  INTO N
  FROM PREUPAIS
  WHERE IDPAIS=PARAM2
  AND IDAPL   =PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_REL;
  END IF;
  -- ACTUALITZA EL PREU DE L'APLICACIO EN EL PAIS DETERMINAT
  UPDATE PREUPAIS
  SET PREU   =PARAM3
  WHERE IDAPL=PARAM1
  AND IDPAIS =PARAM2;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_PREU_NEG THEN
  RSP:='ERROR: PREU INTRODUIT AMB VALOR NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_PAIS_ERR THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR:APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_REL THEN
  RSP:='ERROR: RELACIO APLICACIO-PAIS INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END MOD_PREU_APL;
/
