--------------------------------------------------------
--  DDL for Procedure ACT_USU
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "ACT_USU" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2 )
  -- PROCEDIMENT      ACT_USU
  --
  -- LLISTA LES DESCARREGUES REALITZADES PER UN USUARI IDENTIFICAT PER UN NOMBRE
  -- ENTER (PARAM1)
  --
  -- EL PROCEDIMENT COMPROVA TOTES LES DESCARREGUES QUE HAGI POGUT FER UN USUARI
  -- EN TOTS ELS TERMINALS MOBILS VINCULATS A AQUEST
  --
  -- LES DADES QUE MOSTRARA DE CADA DESCARREGA SN:
  --
  -- NOM DE L'APLICACIO, DATA DE DESCARREGA, PREU PAGAT, MODE DE PAGAMENT
  -- D'AQUESTA, EL DISPOSITIU EN EL QUAL HI HA L'APLICACIO DESCARREGADA, MODEL
  -- D'AQUEST, EL SISTEMA OPERATIU DEL DISPOSITIU, LA MIDA QUE OCUPA I SI
  -- ACTUALMENT EST INSTALADA
  --
  -- PARAMETRES D'ENTRADA:  PARAM1 NUMBER (NUMERO USUARI)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: ENTER >0 NO POT SER NULL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  --ERRORS POSSIBLES:
  --
  --                 VALORS NULS ALS PARAMETRES D'ENTRADA
  --                 USUARI ERRONI O INEXISTENT
  --                 NO HI HA DESCARREGUES REALITZADES PER L'USUARI
  --                 ALTRES
  --
AS
  --DEFINICIONS DELS CURSORS AMB LES CONSULTES A REALITZAR
  --SELECCIONEM LES DADES DE LES DESCARREGUES FETES PER L'USUARI
  CURSOR D
  IS
    SELECT DISTINCT APLICACIO.NOMAPL,
      DESCARREGA.DATDES,
      DESCARREGA.PREU,
      MODePAG.DESP,
      DISPOSITIU.IMEI,
      DISPOSITIU.MODELMOB,
      SO.NOMSO,
      descarrega.inst,
      soapl.mida
    FROM DISPOSITIU,
      MODEL,
      DESCARREGA,
      SO,
      SOAPL,
      APLICACIO,
      USUARI,
      modepag
    WHERE APLICACIO.IDAPL = DESCARREGA.IDAPL
    AND MODEPAG.MODP      =DESCARREGA.MODEP
    AND USUARI.NUMMOB     = DISPOSITIU.NUMMOB
    AND DESCARREGA.IMEI   =DISPOSITIU.IMEI
    AND MODEL.IDSO        =SO.IDSO
    AND SOAPL.IDAPL       =DESCARREGA.IDAPL
    AND SOAPL.IDSO        =MODEL.IDSO
    AND MODEL.MODELMOB    =DISPOSITIU.MODELMOB
    AND USUARI.NUMMOB     =PARAM1;
  -- VARIABLES TEMPORALS
  TAPL APLICACIO.NOMAPL%TYPE;
  TDATA DESCARREGA.DATDES%TYPE;
  TMIDA SOAPL.MIDA%TYPE;
  TPREU DESCARREGA.PREU%TYPE;
  TMODEPAG MODEPAG.DESP%TYPE;
  TDISPOSITIU DISPOSITIU.IMEI%TYPE;
  TMODEL DISPOSITIU.MODELMOB%TYPE;
  TSO SO.NOMSO%TYPE;
  TINST NVARCHAR2(10);
  N NUMBER;
  --EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_USU  EXCEPTION;
  ex_no_des  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'ACT_USU';
BEGIN
  --ES DESA EL PARAMETRE D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- ES COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SN NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- ES COMPROVA SI HI HAN DESCARREGUES DE DISPOSITIUS DE L'USUARI
  -- DESANT EN LES VARIABLES TEMPORALS LES DADES QUE NECESITEM
  OPEN D;
  FETCH d
  INTO TAPL,
    TDATA ,
    TPREU,
    TMODEPAG,
    TDISPOSITIU ,
    TMODEL ,
    TSO,
    tinst,
    TMIDA;
  -- SI NO S'EN TROBEN ES COMPROVA EL MOTIU: O B L'USUARI NO EXISTEIX, O BE NO HA DESCARREGAT CAP APLICACIO
  IF D%NOTFOUND THEN
    -- ES COMPROVA QUE EXISTEIX L'USUARI
    SELECT COUNT(*)
    INTO N
    FROM USUARI
    WHERE NUMMOB=PARAM1;
    IF (N       =0)THEN
      RAISE EX_NO_USU;
      -- SI EXISTEIX IMPLICA QUE NO HI HA DESCARREGUES FETES PER L'USUARI
    ELSE
      raise ex_no_des;
    END IF;
  ELSE
    -- S'HAN TROBAT DESCARREGUES, ES MOSTREN PER PANTALLA
    DBMS_OUTPUT.PUT_LINE('LES APLICACIONS DESCARREGADES PER L''USUARI ' || PARAM1 ||' SN:' );
    DBMS_OUTPUT.PUT_LINE(' ');
    --MENTRES HI HAGI DESCARREGUES ANEM MOSTRAN LES DADES DE CADA DESCARREGA
    WHILE D%FOUND
    LOOP
      -- ES COMPROVA SI L'APLICACIO EN QUESTIO ESTA INSTALADA ACTUALMENT I
      -- ES CANVIA LA VARIABLE UTILITZADA EN LA BASE DE DADES PEL SEU SIGNIFICAT
      IF (TINSt='I')THEN
        TINST :='SI';
      ELSE
        TINST:='NO';
      END IF;
      DBMS_OUTPUT.PUT_LINE('APLICACI: '||TAPL ||' MIDA: '||TMIDA||' DATA DESCARREGADA: '||TDATA ||' PREU PAGAT: '||TPREU ||' MODE PAGAMENT: '||TMODEPAG ||' DISPOSITIU: '||TDISPOSITIU ||' MODEL:'||TMODEL ||' SISTEMA OPERATIU: '||TSO ||' INSTALADA ACTUALMENT: '||TINST);
      --ES DESA ELS SEGUENT ELEMENT A LES VARIABLES TEMPORALS
      FETCH d
      INTO TAPL,
        TDATA ,
        TPREU,
        TMODEPAG,
        TDISPOSITIU ,
        TMODEL ,
        TSO,
        tinst,
        tmida;
    END LOOP;
  END IF;
  --SI S'HA ARRIBAT AQUEST PUNT VOL DIR QUE NO S'HA PRODUIT CAP ERROR DURANT L'EXECUCIO
  --PER TANT GUADEM A LA VARIABLE DE SORTIDA RSP EL TEXT 'OK'
  RSP:='OK';
  --GUARDEM A LA TAULA DE LOG ELS VALORS DE L'EXECUCIO DEL PROCEDIMENT
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  CLOSE D;
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  DBMS_OUTPUT.PUT_LINE(RSP);
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_USU THEN
  RSP:='ERROR: USUARI ERRONI O INEXISTENT';
  DBMS_OUTPUT.PUT_LINE(RSP);
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: NO HI HA APLICACIONS DESCARREGADES PER L''USUARI';
  DBMS_OUTPUT.PUT_LINE(RSP);
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT ACT_USU - '||SQLCODE||'-'||SQLERRM;
  DBMS_OUTPUT.PUT_LINE(RSP);
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END ACT_USU;
/
