--------------------------------------------------------
--  DDL for Procedure APL_ACT
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "APL_ACT" (
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           APL_ACT
  --
  -- AQUEST PROCEDIMENT LLISTA TOTES LES APLICACIONS ACTIVES ORDENADES PER
  -- NOMBRE DE DESCARREGUES, DE MAJOR A MENOR.
  --
  -- PRIMER ES CERQUEN AQUELLES APLICACIONS ACTIVES AMB DESCARREGUES I DESPRES
  -- ES MOSTREN LES APLICACIONS ACTIVES RESTANTS QUE NO TENEN DESCARREGUES, AQUESTES
  -- DARRERS ES MOSTRARAN ORDENADES PER ORDRE DE CREACIO (DE MES ANTIGUES A MES NOVES).
  --
  -- PARAMETRES D'ENTRADA:
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  NO HI HA APLICACIONS ACTIVES
  --                  ALTRES
AS
  -- CURSOR AMB LES CONSULTES A REALITZAR
  -- LLISTAT D'APLICACIONS ACTIVES AMB DESCARREGUES ORDENADES DESCENDENTMENT PER
  -- NOMBRE DE DESCARREGUES
  CURSOR E
  IS
    SELECT descarrega.IDAPL,
      COUNT(*) AS TOTAL
    FROM DESCARREGA,
      APLICACIO
    WHERE DESCARREGA.INST='I'
    AND APLICACIO.IDAPL  =DESCARREGA.IDAPL
    AND APLICACIO.ACTIVA ='A'
    GROUP BY descarrega.IDAPL
    ORDER BY TOTAL DESC;
  -- LLISTAT D'APLICACIONS ACTIVES SENSE DESCARREGUES ORDENADES ASCENDENTMENT PER
  -- NUM DE IDENTIFICACIO, PER TANT DE MS VELLA A MS NOVA.
  CURSOR D
  IS
    SELECT APLICACIO.IDAPL
    FROM APLICACIO
    WHERE APLICACIO.ACTIVA   ='A'
    AND APLICACIO.IDAPL NOT IN
      (SELECT APLICACIO.IDAPL
      FROM DESCARREGA,
        APLICACIO
      WHERE DESCARREGA.INST='I'
      AND APLICACIO.IDAPL  =DESCARREGA.IDAPL
      AND APLICACIO.ACTIVA ='A'
      )
  ORDER BY APLICACIO.IDAPL ASC;
  -- VARIABLES TEMPORALS
  IDAP NUMBER;
  NDES NUMBER;
  TIDAPL APLICACIO.IDAPL%TYPE;
  TNOMAPL APLICACIO.NOMAPL%TYPE;
  TVERSIO APLICACIO.VERSIO%TYPE;
  TMINRESX APLICACIO.MINRESX%TYPE;
  TMINRESY APLICACIO.MINRESY%TYPE;
  TVIDEODEMO APLICACIO.VIDEODEMO%TYPE;
  TDATAUP APLICACIO.DATAUP%type;
  -- EXCEPCIONS
  EX_NO_ACT EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'APL_ACT';
BEGIN
  -- CONSULTEM SI HI HA APLICACIONS ACTIVES
  SELECT COUNT(*)
  INTO NDES
  FROM APLICACIO
  WHERE ACTIVA='A';
  IF (NDES    =0) THEN
    RAISE EX_NO_ACT;
  END IF;
  -- CONSULTEM APLICACIONS AMB DESCARREGUES I ANEM MOSTRANT-LES PER PANTALLA
  OPEN E;
  FETCH E INTO IDAP, NDES;
  IF E%NOTFOUND THEN
    NULL;
  ELSE
    WHILE E%FOUND
    LOOP
      SELECT IDAPL,
        NOMAPL,
        VERSIO,
        MINRESX,
        MINRESY,
        VIDEODEMO,
        DATAUP
      INTO TIDAPL,
        TNOMAPL,
        TVERSIO,
        TMINRESX,
        TMINRESY,
        TVIDEODEMO,
        TDATAUP
      FROM APLICACIO
      WHERE IDAPL =IDAP
      AND ACTIVA  ='A' ;
      DBMS_OUTPUT.PUT_LINE( 'CODI APLICACIO: '||TIDAPL||' NOM APLICACIO: '||TNOMAPL||' VERSIO: '|| TVERSIO||' RESOLUCIO: '|| TMINRESX||'-'||TMINRESY||' URL VIDEO DEMOSTRACIO: '||TVIDEODEMO||' DATA CARREGA: '||TDATAUP||' N DESCARREGUES: '||NDES);
      FETCH E INTO IDAP, NDES;
    END LOOP;
  END IF;
  CLOSE E;
  OPEN D;
  --CONSULTEM APLICACIONS SENSE DESCARREGUES I LES ANEM MOSTRANT PER PANTALLA
  FETCH D
  INTO IDAP;
  IF D%NOTFOUND THEN
    NULL;
  ELSE
    WHILE D%FOUND
    LOOP
      SELECT IDAPL,
        NOMAPL,
        VERSIO,
        MINRESX,
        MINRESY,
        VIDEODEMO,
        DATAUP
      INTO TIDAPL,
        TNOMAPL,
        TVERSIO,
        TMINRESX,
        TMINRESY,
        TVIDEODEMO,
        TDATAUP
      FROM APLICACIO
      WHERE IDAPL =IDAP
      AND ACTIVA  ='A' ;
      DBMS_OUTPUT.PUT_LINE( 'CODI APLICACIO: '||TIDAPL||' NOM APLICACIO: '||TNOMAPL||' VERSIO: '|| TVERSIO||' RESOLUCIO: '|| TMINRESX||'-'||TMINRESY||' URL VIDEO DEMOSTRACIO: '||TVIDEODEMO||' DATA CARREGA: '||TDATAUP||' N DESCARREGUES: '||0);
      FETCH D INTO IDAP;
    END LOOP;
  END IF;
  CLOSE D;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_NO_ACT THEN
  RSP:='ERROR: NO HI HA APLICACIONS ACTIVES';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT APL_ACT - '||SQLCODE||'-'||SQLERRM;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END APL_ACT;
/
