--------------------------------------------------------
--  DDL for Procedure CONS_IDENT_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "CONS_IDENT_APL" (
    PARAM1 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT            CONS_IDENT_APL
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA EL CODI DE IDENTIFICACIO DE L'APLICACIO
  -- LA QUAL TE COM A NOM EL PARAMETRE INTRODUIT (PARAM1)
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NVARCHAR2 (NOM APLICACIO)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS PARAM1: CADENA DE CARACTERS, NOT NULS
  --
  -- VALORS RSP: ERROR + MOTIU DE L'ERROR
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALOR NULS
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'CONS_IDENT_APL';
BEGIN
  -- DESA LA VARIABLE D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF (PARAM1 IS NULL)THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE L'APLICACIO EXISTEIX
  SELECT IDAPL
  INTO N
  FROM APLICACIO
  WHERE NOMAPL=PARAM1;
  -- MOSTRA PER PANTALLA EL CODI DE L'APLICACIO INTRODUIDA
  DBMS_OUTPUT.PUT_LINE('L''APLICACI '||PARAM1||' S''IDENTIFICA AMB EL NM. '||N);
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:= 'ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN NO_DATA_FOUND THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  --GUARDEM  A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END CONS_IDENT_APL;
/
