--------------------------------------------------------
--  DDL for Procedure CON_ID_DESENV
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "CON_ID_DESENV" (
    PARAM1 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           CON_ID_DESENV
  --
  -- AQUEST PROCEDIMENT CERCA EL DESENVOLUPADOR QUE TE PER NOM EL PARAMETRE
  -- INTRODUIT I RETORNA EL CODI DE IDENTIFICACIO DEL DESENVOLUPADOR
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NVARCHAR2 (NOM DESENVOLUPADOR)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: CADENA DE CARACTERS, NOT NULL
  --
  -- VALORS DE RSP:  'OK'
  --                 'ERROR'+ EL MOTIU D'AQUEST
  --
  -- ERRORS POSSIBLES:
  --
  --                  PARAMETRES D'ENTRADA AMB VALORS NULS
  --                  DESENVOLUPADOR ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  DES DESENVOLUPADOR.IDDES%type;
  --EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_DES  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'CON_ID_DESENV';
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- DESENVOLUPADOR AMB EL NOM INTRODUIT
  CURSOR DESENV
  IS
    SELECT IDDES FROM DESENVOLUPADOR WHERE NOMDES=PARAM1;
BEGIN
  -- DESA LA VARIABLE D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SIGUIN NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA EL DESENVOLUPADOR AMB EL NOM INTRODUIT
  OPEN DESENV;
  FETCH DESENV INTO DES;
  -- SI NO EXISTEIX
  IF (DESENV%NOTFOUND) THEN
    RAISE EX_NO_DES;
  END IF;
  CLOSE DESENV;
  -- MOSTRA PER PANTALLA EL NUMERO DE IDENTIFICACIO DEL DESENVOLUPADOR INTRODUIT
  DBMS_OUTPUT.PUT_LINE('EL NUM. D''IDENTIFICACI DEL DESENVOLUPADOR '||PARAM1||' S: '||DES);
  -- S'HA EXECUTAT CORRECTAMENT EL PROCEDIMENT
  RSP:='OK';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_DES THEN
  RSP:='ERROR: DESENVOLUPADOR ERRONI O INEXISTENT';
  --GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDEM A LA TAULA DE LOG
  INSERT
  INTO pLOG VALUES
    (
      procediment,
      entrada,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END CON_ID_DESENV;
/
