--------------------------------------------------------
--  DDL for Procedure DES_APL_PAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "DES_APL_PAIS" (
    param1 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDURE           DES_APL_PAIS
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA TOTS ELS DESENVOLUPADORS QUE
  -- PRTANYEN AL PAIS INTRODUIT I EL NOMBRE DE D'APLICACIONS PUBLICADA PER
  -- CADASCUN D'ELLS. EN EL CAS QUE NO HI HAGI DESENVOLUPADORS HO NOTIFICA
  --
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NVARCHAR2 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS DE RSP:  'OK'
  --                 'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  PAIS ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- CONSULTAR DESENVOLUPADORS DEL PAIS
  CURSOR B
  IS
    SELECT iddes,
      nomdes,
      NOMREP,
      TELDES,
      ADRECADES
    FROM DESENVOLUPADOR
    WHERE IDPAIS = PARAM1;
  -- VARIABLES TEMPORALS
  CE NVARCHAR2(2);
  N NUMBER;
  DESEN DESENVOLUPADOR.IDDES%TYPE;
  NDESEN DESENVOLUPADOR.NOMDES%TYPE;
  NREP DESENVOLUPADOR.NOMREP%TYPE;
  TDES DESENVOLUPADOR.TELDES%TYPE;
  ADR DESENVOLUPADOR.ADRECADES%type;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'DES_APL_PAIS';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:=PARAM1;
  -- COMPROVA QUE ELS PARAMETRES D'ENTRADA NO SON NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA QUE EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM1;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- CONSULTAR DESENVOLUPADORS DEL PAIS
  OPEN B;
  FETCH B INTO DESEN,NDESEN,NREP,TDES,ADR;
  -- NO HI HA DESENVOLUPADORS
  IF B%NOTFOUND THEN
    DBMS_OUTPUT.PUT_LINE('PAIS SENSE DESENVOLUPADORS');
  ELSE
    -- ES MOSTREN ELS DESENVOLUPADORS
    DBMS_OUTPUT.PUT_LINE('S''HAN TROBAT ELS SEGENTS DESENVOLUPADORS:');
    DBMS_OUTPUT.PUT_LINE(' ');
    WHILE B%FOUND
    LOOP
      SELECT COUNT(*) INTO N FROM DESENVOLUPADA WHERE IDDES= DESEN;
      DBMS_OUTPUT.PUT_LINE('DESENVOLUPADOR: '||ndesen||', REPRESENTANT: '||nrep||', TELEFON: '|| tdes||', Adrea: '||adr||', ND''APLICACIONS DESENVOLUPADES: '||n);
      FETCH B INTO DESEN,ndesen,nrep,tdes,adr;
    END LOOP;
  END IF;
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR:  VALORS NULS ALS PARAMETRES D''ENTRADA';
  --GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  --GUARDA  A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  --GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END DES_APL_PAIS;
/
