--------------------------------------------------------
--  DDL for Procedure PAIS_APL
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "PAIS_APL" (
    param1 IN NUMBER,
    param2 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT                PAIS_APL
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA EL LLISTAT DE TOTS ELS PAISOS ON
  -- AQUESTA S'HA DESCARREGAT EN L'ANY INDICAT I EL NOMBRE DE DESCARREGUES DE
  -- CADA PAIS, EN CAS QUE NO HI HAGI DESCARREGUES AQUELL ANY EN AQUELL PAIS
  -- HO INDICA PER PANTALLA
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (CODI APLICACIO), PARAM2 NUMBER (ANY)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS PARAM1: NUMBER>0, NO NUL
  --
  -- VALORS PARAM2: NUMBER>0 I NUMERO<=ANY ACTUAL, NO NUL
  --
  -- VALORS RSP:    'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  APLICACIO ERRONIA O INEXISTENT
  --                  ANY POSTERIOR A L'ACTUAL O NEGATIU
  --                  ALTRES
AS
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- APLICACIO AMB CODI 'N'
  CURSOR NAPL
  IS
    SELECT NOMAPL FROM APLICACIO WHERE IDAPL=PARAM1;
  -- SELECCIONA LES DESCARREGUES INSTALADES D'UNA APLICACIO I AGRUPALES PER PAIS
  CURSOR B
  IS
    SELECT COUNT(*),
      PAIS.NOMPAIS
    FROM PAIS,
      DESCARREGA
    WHERE DESCARREGA.IDAPL=param1
    AND param2            =EXTRACT(YEAR FROM DESCARREGA.DATDES)
    AND PAIS.IDPAIS       =DESCARREGA.IDPAIS
    AND DESCARREGA.INST   ='I'
    GROUP BY PAIS.NOMPAIS ;
  -- VARIABLES TEMPORALS
  TCONT NUMBER;
  TPAIS PAIS.NOMPAIS%TYPE;
  TNOMAP APLICACIO.NOMAPL%TYPE;
  --EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_APL  EXCEPTION;
  EX_NO_ANY  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'PAIS_APL';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2;
  -- COMPROVA SI HI HA VALORS NULS ALS PARAMETRES D'ENTRADA
  IF ((PARAM1 IS NULL)OR (PARAM2 IS NULL)) THEN RAISE EX_VAL_NUL;END IF;
  -- COMPROVA SI L'ANY ESTA DINS DE RANG
  IF ((PARAM2<0) OR (PARAM2>extract(year from CURRENT_dATE))) THEN  RAISE EX_NO_ANY; END IF;
  -- CONSULTA EL NOM DE L'APLICACIO
  OPEN NAPL;
  FETCH NAPL INTO TNOMAP;
  IF (NAPL%NOTFOUND) THEN
    RAISE EX_NO_APL;
  END IF;
  -- CONSULTA LES DESCARREGUES INSTALADES DE L'APLICACIO, I LES CONTA AGRUPANTLES
  -- PER PAIS
  OPEN B;
  FETCH B INTO TCONT,TPAIS;
  -- SI NO HI HA DESCARREGUES QUE COMPLEIXIN LES CONDICIONS HO DIU PER PANTALLA
  IF (B%NOTFOUND)THEN
    DBMS_OUTPUT.PUT_LINE('NO HI HA DESCARREGUES DE L''APLICACIO '||TNOMAP||'  L''ANY '||PARAM2);
    -- SI HI HA DESCARREGUES EN MOSTRA LES DADES TROBADES
  ELSE
    DBMS_OUTPUT.PUT_LINE('LES DESCARREGUES A NIVELL MUNDIAL DE L''APLICACIO '||TNOMAP||'  L''ANY '||PARAM2);
    DBMS_OUTPUT.PUT_LINE(' ');
    WHILE (B%FOUND)
    LOOP
      SYS.DBMS_OUTPUT.PUT_LINE('PAIS: '||TPAIS||'   N DE DESCARREGUES:  '||TCONT);
      FETCH B INTO TCONT,TPAIS;
    END LOOP;
  END IF;
  --S'HA EXECUTAT EL PROCEDIMENT SENSE ERRORS
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:= 'ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY THEN
  RSP:= 'ERROR: ANY INTRODUIT POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_APL THEN
  RSP:='ERROR: APLICACIO ERRONIA O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END PAIS_APL;
/
