--------------------------------------------------------
--  DDL for Procedure TOP_20
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "TOP_20" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT           TOP_20
  --
  -- AQUEST PROCEDIMENT MOSTRA EL LLISTAT DELS 20 USUARIS QUE MES S'HAN GASTAT,
  -- EN UN ANY DETERMINAT, EN APLICACIONS, EN EL CAS QUE NO ARRIBIN A SER 20
  -- ELS USUARIS MOSTRARA AQUELLS QUE HI HAGI I EN EL CAS QUE NO HI HAGI USUARIS
  -- QUE HAGIN DESCARREGAT EN AQUELL ANY HO NOTIFICARA.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0 I NUMBER<= ANY ACTUAL, NO NUL
  --
  -- VALORS DE RSP:    'OK'
  --                   'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A LA DATA ACTUAL O NEGATIU
  --                  ALTRES
AS
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- LLISTAR ELS USUARIS QUE HAN DESGARREGAT L'ANY INDICAT, N DE DESCARREGUES I LA
  -- SUMA DELS PREUS D'AQUESTES. ORDENAT DE MAJOR SUMA A MENOR
  CURSOR b
  IS
    SELECT USUARI.NUMMOB,
      COUNT(*),
      SUM(DESCARREGA.PREU )AS TOTAL
    FROM USUARI,
      DESCARREGA,
      DISPOSITIU
    WHERE usuari.nummob                     =dispositiu.nummob
    AND DESCARREGA.IMEI                     =DISPOSITIU.IMEI
    AND EXTRACT(YEAR FROM DESCARREGA.DATDES)=param1
    GROUP BY USUARI.NUMMOB
    ORDER BY TOTAL DESC;
  -- VARIABLES TEMPORALS
  US USUARI.NUMMOB%TYPE;
  AP NUMBER;
  GAS DESCARREGA.PREU%TYPE;
  CONT NUMBER DEFAULT 0;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_ANY  EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'TOP_20';
BEGIN
  --ES DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  -- ES COMPROVA QUE CAP PARAMETRE D'ENTRADA ES NULL
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  end if;
  -- ES COMPROVA SI L'ANY INTRODUIT ESTA DINS DEL RANG
   IF ((PARAM1>extract(YEAR FROM CURRENT_DATE))OR(PARAM1<=0))THEN
    RAISE EX_NO_ANY;
  END IF;
  -- ES CONSULTA ELS USUARIS AMB DESCARREGUES AQUELL ANY, LA QUANTITAT I EL PREU
  -- TOTAL D'ELLES PER CADA USUARI.
  OPEN B;
  FETCH B INTO US,AP,GAS;
  -- NO HI HA DESCARREGUES INSTALADES AQUELL ANY PER TANT NO HI HA USUARIS QUE 
  -- LES HAGIN DESCARREGAT
  IF (B%NOTFOUND) THEN
    DBMS_OUTPUT.PUT_LINE('NO HI HA DESCARREGUES REALITZADES AQUELL ANY');
    --HI HA DESCARREGUES PER TANT USUARIS QUE LES HAN DESCCARREGAT
  ELSE
   -- LES MOSTREM PER PANTALLA 
   DBMS_OUTPUT.PUT_LINE('ELS 20 USUARIS QUE S''HAN GASTAT MS L''ANY '||PARAM1||' SON: ');
      DBMS_OUTPUT.PUT_LINE(' ');
    WHILE ((B%FOUND) AND (CONT<20))
    LOOP
      CONT:=CONT+1;
      DBMS_OUTPUT.PUT_LINE('USUARI: '||US||' NDESCARREGUES: '||AP||' TOTAL GASTAT: '||GAS);
      FETCH B INTO US,AP,GAS;
    END LOOP;
  END IF;
 --S'HA EXECUTAT EL PROCEDIMENT SENSE ERRORS
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  commit;
  WHEN EX_NO_ANY THEN
  RSP:='ERROR: ANY POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;

WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END TOP_20;
/
