--------------------------------------------------------
--  DDL for Function CONTROL_EST
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "CONTROL_EST" (
      PARAM1 IN NUMBER ,
      PARAM2 IN NVARCHAR2 )
    RETURN NVARCHAR2
    -- FUNCIO              CONTROL_EST
    --
    -- AQUESTA FUNCIO  ES UNA FUNCIO INTERNA QUE S'ENCARREGA DE MANTENIR LES
    -- TAULES ESTADISTIQUES ACTUALITZADES SOBRE L'ANY I EL PAIS QUE S'ENTREN
    -- COM A PARAMETRES
    --
    -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY), PARAM2 NVARCHAR2 (CODI PAIS)
    -- PARAMETRES DE SORTIDA: NVARCHAR2 (RESULTAT DE L'OPERACIO)
    --
    -- VALORS DE PARAM1: NUMBER>0, NO NUL
    --
    -- VALORS DE PARAM2: CADENA DE CARACTER MIDA=2, NO NUL
    --
    -- VALORS DE SORTIDA:'OK'
  AS
    -- VARIABLES TEMPORALS
    N NUMBER;
    D NUMBER;
    F number;
    E NUMBER;
    DESENV DESENVOLUPADOR.IDDES%TYPE;
    -- CURSORS AMB LES CONSULTES A REALITZAR
    -- COMPTAR LES DESCARREGUES FETES D'APLICACIONS AGRUPADES PEL DESENVOLUPADOR
    -- QUE LES DESENVOLUPAVA ORDENADES DESCENDENTMENT
    CURSOR DES
    IS
      SELECT COUNT(*) AS N,
        IDDES
      FROM DESCARREGA,
        DESENVOLUPADA
      WHERE EXTRACT(YEAR FROM DESCARREGA.DATDES)=param1
      AND DESCARREGA.INST                       ='I'
      AND DESENVOLUPADA.IDAPL                   =DESCARREGA.IDAPL
      GROUP BY DESENVOLUPADA.IDDES
      ORDER BY N DESC;
    -- SUMAR EL TOTAL DEL PREU DE LES DESCARREGUES FETES, AGRUPADES PER L'APLICACIO
    CURSOR APL
    IS
      SELECT SUM(PREU) AS N,
        IDAPL
      FROM DESCARREGA
      WHERE EXTRACT(YEAR FROM DESCARREGA.DATDES)=param1
      AND descarrega.inst                       ='I'
      GROUP BY DESCARREGA.IDAPL
      ORDER BY n DESC;
  BEGIN
    -- CONSULTAR SI HI HA JA FETA L'ESTADISTICA ANUAL DE L'ANY INTRODUIT
    SELECT COUNT(*)
    INTO N
    FROM EST_ANUAL
    WHERE EST_ANUAL.ANYC=PARAM1;
    -- SI NO, S'INTRODUEIX EL VALOR D'ESTADISTIQUES D'AQUELL ANY
    IF (N =0)THEN
      INSERT INTO est_anual VALUES
        (PARAM1,1,null,null,null
        ); END IF;
   
      -- ES COMPTA EL NOMBRE DE DESCARREGUES INSTALADES D'AQUELL ANY
      SELECT COUNT(*)
      INTO N
      FROM DESCARREGA
      WHERE EXTRACT(YEAR FROM DATDES)=param1
      AND DESCARREGA.INST            ='I';
      -- ES COMPTA EL NOMBRE D'USUARIS QUE HAN REALITZAT DESCARREGUES AQUELL ANY
      SELECT COUNT(*)
      INTO D
      FROM
        (SELECT DISTINCT NUMMOB
        FROM DESCARREGA,
          DISPOSITIU
        WHERE DISPOSITIU.IMEI                   =DESCARREGA.IMEI
        AND DESCARREGA.INST                     ='I'
        AND EXTRACT(YEAR FROM DESCARREGA.DATDES)=param1
        );
      -- SI EL NOMBRE D'USUARIS ES '0' IMPLICA QUE NO HI HA DESCARREGUES REALITZADES
      -- AQUELL ANY PER TANT LA MITJA ES '0'
      IF (D=0)THEN
        F :=0;
        -- SI HI HA USUARIS VOL DIR QUE HI HA UN MINIM D'UNA DESCARREGA PER TANT
        -- LA MITJA SERA EL NOMBRE DE DESCARREGUES ENTRE ELS USUARIS.
      ELSE
        F:=(N)/(D);
      END IF;
      -- S'ACTUALITZA EL PARAMETRE A LA TAULA D'ESTADISTIQUES ANUALS
      UPDATE EST_ANUAL
      SET MITJADESC       =F
      WHERE EST_ANUAL.ANYC=PARAM1;
      --CONFIRMEM ELS CANVIS FETS .
      COMMIT;
 
    -- COMPROVAR DESENVOLUPADOR MAXIM
    OPEN DES;
    FETCH DES INTO n,DESENV;
    IF (DES%NOTFOUND) THEN
       update EST_ANUAL
      set DESENVOLUPADOR  =null,
        DESDES            =NULL
      WHERE EST_ANUAL.ANYC=param1;
    ELSE
      UPDATE EST_ANUAL
      SET DESENVOLUPADOR  =DESENV,
        DESDES            =N
      WHERE EST_ANUAL.ANYC=param1;
    END IF;
    --CONFIRMEM ELS CANVIS FETS .
    COMMIT;
    CLOSE DES;
    --COMPROVAR L'APLICACIO MAXIMA DE L'ANY
    OPEN APL;
    FETCH APL INTO F,D;
    IF (APL%NOTFOUND) THEN
      UPDATE EST_ANUAL SET APLICACIO=NULL WHERE EST_ANUAL.ANYC=param1;
    ELSE
      UPDATE EST_ANUAL SET APLICACIO=d WHERE EST_ANUAL.ANYC=param1;
      --CONFIRMEM ELS CANVIS FETS .
      COMMIT;
    END IF;
    CLOSE APL;
    -- COMPROVEM SI S'HA INTRODUIT ESTADISTIQUES D'AMBIT ANUAL-LOCAL DEL ANY I
    -- EL PAIS INTRODUITS
    SELECT COUNT(*)
    INTO E
    FROM EST_ANUAL_LOCAL
    WHERE EST_ANUAL_LOCAL.ANYC=param1
    AND EST_ANUAL_LOCAL.PAIS  =PARAM2;
    -- CONSULTEM LA SUMA DELS INGRESOS FETS DURANT L'ANY I EN EL PAIS SELECCIONAT
    SELECT SUM(PREU)
    INTO F
    FROM DESCARREGA
    WHERE extract(YEAR FROM DESCARREGA.DATDES)=PARAM1
    AND DESCARREGA.INST                       ='I'
    AND DESCARREGA.IDPAIS                     =PARAM2;
    IF (F                                    IS NULL) THEN
      F                                      :=0;
    END IF;
    -- CONSULTEM EL NOMBRE D'USUARIS DIFERENTS QUE HAN DESCARREGAT APLICACIONS
    -- L'ANY I EN EL PAIS INDICATS
    SELECT COUNT(*)
    INTO N
    FROM
      (SELECT DISTINCT USUARI.NUMMOB
      FROM USUARI,
        DESCARREGA,
        DISPOSITIU
      WHERE EXTRACT(YEAR FROM DESCARREGA.DATDES)=PARAM1
      AND USUARI.NUMMOB                         =DISPOSITIU.NUMMOB
      AND DESCARREGA.INST                       ='I'
      AND DISPOSITIU.IMEI                       =DESCARREGA.IMEI
      AND DESCARREGA.IDPAIS                     =PARAM2
      );
    -- CONSULTEM EL NOMBRE D'APLICACIONS DIFERENTS QUE S'HAN DESCARREGAT
    -- L'ANY I EN EL PAIS INDICATS
    SELECT COUNT(*)
    INTO D
    FROM
      (SELECT DISTINCT DESCARREGA.IDAPL
      FROM DESCARREGA
      WHERE EXTRACT(YEAR FROM DESCARREGA.DATDES)=PARAM1
      AND DESCARREGA.INST                       ='I'
      AND DESCARREGA.IDPAIS                     =PARAM2
      );
    -- SI NO S'HAN TROBAT ESTADISTIQUES DEL ANY I EL PAIS INTRODUITS INSERTEM UN
    -- NOU REGISTRE AMB LES DADES COMPUTADES
    IF (E=0) THEN
      INSERT INTO EST_ANUAL_LOCAL VALUES
        (param1,PAram2,N,f,D
        );
      --CONFIRMEM ELS CANVIS FETS .
      COMMIT;
      -- SI S'HAN TROBAT ESTADISTIQUES DEL ANY I EL PAIS INTRODUITS MODIFIQUEM
      -- EL REGISTRE AMB LES DADES COMPUTADES
    ELSE
      UPDATE EST_ANUAL_LOCAL
      SET EST_ANUAL_LOCAL.INGRESSOS=f,
        NAPLICACIONS               =D,
        NUSUARIS                   =N
      WHERE EST_ANUAL_LOCAL.ANYC   =param1
      AND EST_ANUAL_LOCAL.PAIS     =PARAM2;
      --CONFIRMEM ELS CANVIS FETS .
      COMMIT;
    END IF;
    RETURN 'OK';
    --TRACTAMENT D'EXCEPCIONS
  EXCEPTION
  WHEN OTHERS THEN
    DBMS_OUTPUT.PUT_LINE('ERROR: SENSE TRACTAR.  - '||SQLCODE||'-'||SQLERRM);
    
  END CONTROL_EST;
/
