--------------------------------------------------------
--  DDL for Procedure EST_APL_ANY_PAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_APL_ANY_PAIS" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT              EST_APL_ANY_PAIS
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA EL NOMBRE D'APLICACIONS DIFERENTS
  -- DESCARREGADES EN AQUEL PAIS EN L'ANY INDICAT.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY), PARAM2 NVARCHAR2 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS PARAM1: NUMBER>0 I NUMBER<=ANY ACTUAL, NO NUL
  --
  -- VALORS PARAM2: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS RSP:  'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY INTRODUIT MAJOR A L'ANY ACTUAL O NEGATIU
  --                  PAIS ERRONI O INEXISTENT
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_ANY_ERR EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'EST_APL_ANY_PAIS';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF (PARAM1 IS NULL) OR (PARAM2 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  --COMPROVA SI L'ANY INTRODUIT ES CORRECTE
  IF ((PARAM1>EXTRACT(YEAR FROM CURRENT_DATE))OR(PARAM1<=0)) THEN
    RAISE EX_ANY_ERR;
  END IF;
  -- COMPROVA SI EXISTEIX EL PAIS
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM2;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- CONSULTA LA TAULA D'ESTADISTIQUES ANUALS I LOCALS CERCANT EL NOMBRE D'APLICACIONS
  -- DESCARREGADES AQUELL ANY I EN AQUELL PAIS
  SELECT NAPLICACIONS
  INTO N
  FROM EST_ANUAL_LOCAL
  WHERE EST_ANUAL_LOCAL.ANYC=PARAM1
  AND PAIS                  =PARAM2;
  -- MOSTRA EL RESULTAT PER PANTALLA
  SYS.DBMS_OUTPUT.PUT_LINE('EL NOMBRE D''APLICACIONS DIFERENTS DESCARREGADES L''ANY '||PARAM1||'EN EL PAIS '||PARAM2||' S: '||N);
  -- S'HA EXECUTAT CORRECTAMENT EL PROCEDIMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
  -- NO HI HA APLICACIONS DESCARREGADES AQUELL ANY I EN AQUELL PAIS
WHEN NO_DATA_FOUND THEN
  -- MOSTREM PER PANTALLA LA MANCA DE RESULTATS.
  SYS.DBMS_OUTPUT.PUT_LINE ('NO HI HA APLICACIONS DESCARREGADES EN L''ANY '||PARAM1||'EN EL PAIS '||PARAM2);
  RSP:='ERROR: NO HI HA APLICACIONS DESCARREGADES EN L''ANY '||PARAM1||'EN EL PAIS '||PARAM2;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS INTRODUITS AMB VALOR NUL';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_ANY_ERR THEN
  RSP:='ERROR: ANY INTRODUIT POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_APL_ANY_PAIS;
/
