--------------------------------------------------------
--  DDL for Procedure EST_APL_MAX
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_APL_MAX" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT             EST_APL_MAX
  --
  -- AQUEST PROCEDIMENT MOSTRA PER PANTALLA LA INFORMACIO DE L'APLICACIO QUE MES
  -- DINERS HA RECAUDAT EN L'ANY INDICAT I ELS DESENVOLUPADORS D'AQUESTA
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0 I NUMBER<=ANY ACTUAL, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A LA DATA ACTUAL O NEGATIU
  --                  NO HI HA APLICACIONS DESCARREGADES AQUELL ANY
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  RETORN NVARCHAR2(1000);
  TTEXT APLICACIO.NOMAPL%type;
  N NUMBER;
  F NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_ANY  EXCEPTION;
  EX_NO_ANY2 EXCEPTION;
  -- CURSORS AMB LES CONSULTES A REALITZAR
  -- DESENVOLUPADORS DE L'APLICACIO MAXIMA DE L'ANY INDICAT
  CURSOR DESENVOLUPADORS
  IS
    SELECT IDDES,
      IDAPL
    FROM DESENVOLUPADA
    WHERE IDAPL=
      (SELECT APLICACIO FROM EST_ANUAL WHERE EST_ANUAL.ANYC=PARAM1
      );
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'EST_APL_MAX';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1;
  --COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF (PARAM1 IS NULL)THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI L'ANY INTRODUIT ESTA DINS DEL RANG CORRECTE
  IF ((PARAM1>extract(YEAR FROM CURRENT_DATE))OR(PARAM1<=0))THEN
    RAISE EX_NO_ANY;
  END IF;
  -- COMPROVA SI L'ANY INTRODUIT ESTA DINS LES ESTADISTIQUES (EN CAS CONTRARI SIGNIFICA
  -- QUE NO S'HA REALITZAT DESCARREGUES AQUELL ANY).
  SELECT COUNT(*)
  INTO N
  FROM EST_ANUAL
  WHERE EST_ANUAL.ANYC=PARAM1;
  IF (N               =0)THEN
    RAISE EX_NO_ANY2;
  END IF;
  -- SELECCIONA EL NOM DE L'APLICACIO TROBADA
  SELECT NOMAPL
  INTO TTEXT
  FROM APLICACIO
  WHERE IDAPL=N;
  -- AFEGEIX A LA VARIABLE TEMPORAL EL NOM D'APLICACIO
  RETORN:='L''APLICACIO AMB MS INGRESSOS L''ANY '||PARAM1||' ES: '||TTEXT||' I ELS SEUS DESENVOLUPADORS SN: ';
  -- AFEGEIX A LA VARIABLE TEMPORAL ELS NOMS DELS DESENVOLUPADORS DE L'APLICACIO
  OPEN DESENVOLUPADORS;
  FETCH DESENVOLUPADORS INTO F,N;
  WHILE (DESENVOLUPADORS%FOUND)
  LOOP
    SELECT NOMDES INTO TTEXT FROM DESENVOLUPADOR WHERE IDDES=F;
    RETORN:=RETORN ||TTEXT||',';
    FETCH DESENVOLUPADORS INTO F,N;
  END LOOP;
  -- RETORNA LA VARIABLE TEMPORAL ON S'HA ANAT AFEGINT L'INFORMACIO REQUERIDA.
  DBMS_OUTPUT.PUT_LINE(RETORN);
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY THEN
  RSP:='ERROR: ANY POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY2 THEN
  RSP:='ERROR: NO HI HA DESCARREGUES EN L''ANY INTRODUIT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_APL_MAX;
/
