--------------------------------------------------------
--  DDL for Procedure EST_AVG_ANY
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_AVG_ANY" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT          EST_AVG_ANY
  --
  -- AQUEST PROCEDIMENT MOSTRA PER PANTALLA EL NUMERO MIG D'APLICACIONS DESCARREGADES
  -- PER UN USUARI, CERCANT LES DADES A LA TAULA D'ESTADISTIQUES D'AMBIT ANUAL.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DE PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NUMER <=ANY ACTUAL, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A LA DATA ACTUAL O NEGATIU
  --                  NO HI HA APLICACIONS DESCARREGADES AQUELL ANY
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N EST_ANUAL.MITJADESC%type;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_ANY  EXCEPTION;
  EX_NO_ANY2 EXCEPTION;
  -- PARAMETRES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'EST_AVG_ANY';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:= PARAM1;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI L'ANY ESTA DINS DEL RANG CORRECTE
  IF (PARAM1>extract(YEAR FROM CURRENT_DATE)OR(PARAM1<0))THEN
    RAISE EX_NO_ANY;
  END IF;
  -- COMPROVA SI HI HA ESTADISTIQUES D'AQUELL ANY
  SELECT COUNT(*)
  INTO N
  FROM EST_ANUAL
  WHERE EST_ANUAL.ANYC=PARAM1;
  IF (N               =0)THEN
    RAISE EX_NO_ANY2;
  END IF;
  -- SELECCIONA LA MITJA DE L'ANY SELECCIONAT I EL MOSTRA PER PANTALLA
  SELECT MITJADESC
  INTO N
  FROM EST_ANUAL
  WHERE EST_ANUAL.ANYC=PARAM1;
  DBMS_OUTPUT.PUT_LINE('LA MITJA DE DESCARREGUES PER USUARI L''ANY '||PARAM1||' S: '||N);
  -- S'HA REALITZAR EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY THEN
  RSP:='ERROR: ANY POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY2 THEN
  RSP:='ERROR: NO HI HA DESCARREGUES EN L''ANY INTRODUIT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_AVG_ANY;
/
