--------------------------------------------------------
--  DDL for Procedure EST_DES_ANY
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_DES_ANY" (
    PARAM1 IN NUMBER,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT            EST_DES_ANY
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA EL DESENVOLUPADOR QUE TE EL MAXIM NUMERO
  -- DE DESCARREGUES I AQUEST NOMBRE CERCANT A LA TAULA D'ESTADISTIQUES D'AMBIT
  -- ANUAL
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DE PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER>0, NUMER <=ANY ACTUAL, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A LA DATA ACTUAL O NEGATIU
  --                  NO HI HA APLICACIONS DESCARREGADES AQUELL ANY
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N DESENVOLUPADOR.NOMDES%type;
  D EST_ANUAL.DESDES%type;
  --EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_NO_ANY  EXCEPTION;
  EX_NO_ANY2 EXCEPTION;
  -- PARAMETRES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%type;
  PROCEDIMENT PLOG.PROCEDIMENT%type DEFAULT 'EST_DES_ANY';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:= PARAM1;
  -- COMPROVA SI ELS PARAMETRES D'ENTRADA SON NULS
  IF (PARAM1 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI L'ANY INTRODUIT ESTA DINS DEL RANG
  IF ((PARAM1>extract(YEAR FROM CURRENT_DATE))OR(PARAM1<=0))THEN
    RAISE EX_NO_ANY;
  END IF;
  -- COMPROVA SI HI HAN ESTADISTIQUES D'AQUELL ANY
  SELECT COUNT(*)
  INTO N
  FROM EST_ANUAL
  WHERE EST_ANUAL.ANYC=PARAM1;
  IF (N               =0)THEN
    RAISE EX_NO_ANY2;
  END IF;
  -- CONSULTA EL NOM DEL DESENVOLUPADOR I ELS VALOR DE LA TAULA D'ESTADISTIQUES
  -- I LES MOSTRA PER PANTALLA
  SELECT DESENVOLUPADOR.NOMDES,
    DESDES
  INTO N,
    D
  FROM EST_ANUAL,
    DESENVOLUPADOR
  WHERE EST_ANUAL.ANYC    =PARAM1
  AND DESENVOLUPADOR.IDDES=EST_ANUAL.DESENVOLUPADOR;
  DBMS_OUTPUT.PUT_LINE('EL DESENVOLUPADOR: '||N||' AMB '||D||' DESCARRREGUES ES EL MAXIM DESENVOLUPADOR L''ANY '||PARAM1);
  -- S'HA REALITZAT L'EXECUCIO DEL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY THEN
  RSP:='ERROR: ANY POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_ANY2 THEN
  RSP:='ERROR: NO HI HA DESCARREGUES EN L''ANY INTRODUIT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:='ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' - '||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_DES_ANY;
/
