--------------------------------------------------------
--  DDL for Procedure EST_ING_ANY_PAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_ING_ANY_PAIS" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT          EST_ING_ANY_PAIS
  --
  -- AQUEST PROCEDIMENT RETORNA PER PANTALLA ELS INGRESOS FETS EN UN PAIS I EN UN
  -- ANY CONCRET, CONSULTANT LA TAULA D'ESTADISTIQUES D'AMBIT ANUAL I LOCAL.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY), PARAM2 NVARCHAR (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER >0 I NUMBER <=ANY ACTUAL, NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A L'ACTUAL O NEGATIU
  --                  PAIS ERRONI O INEXISTENT
  --                  NO HI HA APLICACIONS DESCARREGADES AQUELL ANY I EN AQUELL PAIS
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_ANY_ERR EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'EST_ING_ANY_PAIS';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA:=PARAM1||','||PARAM2;
  -- COMPROVA SI HI HA VALORS NULS ALS PARAMETRES D'ENTRADA
  IF (PARAM1 IS NULL) OR (PARAM2 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI L'ANY ENTRA DINS DEL RANG CORRECTE
  IF ((PARAM1>EXTRACT(YEAR FROM CURRENT_DATE))OR(PARAM1<=0)) THEN
    RAISE EX_ANY_ERR;
  END IF;
  -- COMPROVA SI EXISTEIX EL PAIS INTRODUIT
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM2;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- CONSULTA ELS INGRESSOS LA TAULA DE ESTADISTIQUES (ANUALS I LOCALS) I MOSTRA ELS
  -- RESULTATS PER PANTALLA
  SELECT INGRESSOS
  INTO N
  FROM EST_ANUAL_LOCAL
  WHERE EST_ANUAL_LOCAL.ANYC=PARAM1
  AND PAIS                  =PARAM2;
  SYS.DBMS_OUTPUT.PUT_LINE('ELS INGRESSOS DE L''ANY '||PARAM1||'EN EL PAIS '||PARAM2||' S: '||N);
  -- S'HA REALITZAR EL PROCEDIMENT SENSE ERRORS
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  -- TRACTAMENT D'EXCEPCIONS
EXCEPTION
  -- NO HI HA DATES ESTADISTIQUES
WHEN NO_DATA_FOUND THEN
  RSP:='ERROR: NO HI HA APLICACIONS DESCARREGADES EN L''ANY '||PARAM1||'EN EL PAIS '||PARAM2;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS INTRODUITS AMB VALOR NUL';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_ANY_ERR THEN
  RSP:='ERROR: ANY INTRODUIT POSTERIOR A LA DATA ACTUAL O NEGATIU';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_ING_ANY_PAIS;
/
