--------------------------------------------------------
--  DDL for Procedure EST_USU_ANY_PAIS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "EST_USU_ANY_PAIS" (
    PARAM1 IN NUMBER,
    PARAM2 IN NVARCHAR2,
    RSP OUT NVARCHAR2)
  -- PROCEDIMENT            EST_USU_ANY_PAIS
  --
  -- AQUEST PROCEDIMENT MOSTRA PER PANTALLA EL NOMBRE D'USUARI DIFERENTS QUE
  -- HAN FET UN MINIM D'UNA DESCARREGA CONSULTANT AQUEST VALOR A LA TAULA
  -- D'ESTADISTIQUES ANUALS I LOCALS.
  --
  -- PARAMETRES D'ENTRADA: PARAM1 NUMBER (ANY), PARAM2 NVARCHAR2 (CODI PAIS)
  -- PARAMETRES DE SORTIDA: RSP NVARCHAR2 (RESULTAT DE L'EXECUCIO DEL PROCEDIMENT)
  --
  -- VALORS DE PARAM1: NUMBER >0 I NUMBER <=ANY ACTUAL, NO NUL
  --
  -- VALORS DE PARAM2: CADENA DE CARACTERS, MIDA=2, NO NUL
  --
  -- VALORS DE RSP: 'OK'
  --                'ERROR'+ EL MOTIU D'AQUEST
  --
  -- POSSIBLES ERRORS:
  --
  --                  VALORS NULS ALS PARAMETRES D'ENTRADA
  --                  ANY POSTERIOR A L'ACTUAL O NEGATIU
  --                  PAIS ERRONI O INEXISTENT
  --                  NO HI HA USUARIS AMB DESCARREGUES AQUELL ANY I EN AQUELL PAIS
  --                  ALTRES
AS
  -- VARIABLES TEMPORALS
  N NUMBER;
  -- EXCEPCIONS
  EX_VAL_NUL EXCEPTION;
  EX_ANY_ERR EXCEPTION;
  EX_NO_PAIS EXCEPTION;
  -- VARIABLES PER A LA TAULA DE LOG
  ENTRADA PLOG.ENTRADA%TYPE;
  PROCEDIMENT PLOG.PROCEDIMENT%TYPE DEFAULT 'EST_USU_ANY_PAIS';
BEGIN
  -- DESA ELS PARAMETRES D'ENTRADA PER A LA TAULA DE LOG
  ENTRADA :=PARAM1||','||PARAM2;
  -- COMPROVA SI SON NULS ELS PARAMETRES D'ENTRADA
  IF (PARAM1 IS NULL) OR (PARAM2 IS NULL) THEN
    RAISE EX_VAL_NUL;
  END IF;
  -- COMPROVA SI L'ANY ESTA DINS DEL RANG
  IF ((PARAM1>EXTRACT(YEAR FROM CURRENT_DATE))OR(PARAM1<=0)) THEN
    RAISE EX_ANY_ERR;
  END IF;
  -- COMPROVA SI EL PAIS EXISTEIX
  SELECT COUNT(*)
  INTO N
  FROM PAIS
  WHERE IDPAIS=PARAM2;
  IF (N       =0) THEN
    RAISE EX_NO_PAIS;
  END IF;
  -- CONSULTA A LA TAULA D'ESTADISTIQUES ANUALS-LOCALS EL NOMBRE D'USUARIS DIFERENTS
  -- DEL PAIS SELECCIONAT AMB UNA DESCARREGA COM A MINIM FETA AQUELL ANY I HO MOSTRA
  -- PER PANTALLA
  SELECT NUSUARIS
  INTO N
  FROM EST_ANUAL_LOCAL
  WHERE EST_ANUAL_LOCAL.ANYC=PARAM1
  AND PAIS                  =PARAM2;
  SYS.DBMS_OUTPUT.PUT_LINE('ELS USUARIS QUE HAN REALITZAT UN MININ D''UNA DESCARREGA L''ANY '||PARAM1||'EN EL PAIS '||PARAM2||' SON: '||N);
  -- S'HA EXECUTAT EL PROCEDIMENT CORRECTAMENT
  RSP:='OK';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
  --TRACTAMENT D'EXCEPCIONS
EXCEPTION
  -- NO HI HA DADES A LA TAULA D'ESTADISTIQUES
WHEN NO_DATA_FOUND THEN
  RSP:='ERROR: NO HI HA USUARIS AMB DESCARREGUES L''ANY '||PARAM1||'EN EL PAIS '||PARAM2;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_VAL_NUL THEN
  RSP:='ERROR: VALORS NULS ALS PARAMETRES D''ENTRADA';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_ANY_ERR THEN
  RSP:='ERROR: ANY INTRODUIT POSTERIOR A LA DATA ACTUAL';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN EX_NO_PAIS THEN
  RSP:='ERROR: PAIS ERRONI O INEXISTENT';
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
WHEN OTHERS THEN
  RSP:= 'ERROR: SENSE TRACTAR. PROCEDIMENT '||PROCEDIMENT||' -'||SQLCODE||'-'||SQLERRM;
  -- GUARDA A LA TAULA DE LOG
  INSERT
  INTO PLOG VALUES
    (
      PROCEDIMENT,
      ENTRADA ,
      RSP,
      CURRENT_DATE
    );
  --CONFIRMEM ELS CANVIS FETS A LA BASE DE DADES DURANT L'EXECUCIO DEL PROCEDIMENT.
  COMMIT;
END EST_USU_ANY_PAIS;
/
